"""This module is for the translation mapping data of the
following document:

Document file: dictionary_generic.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Dictionary class generic type settings':
    '',

    'This page explains the `Dictionary` class key and value\'s generic type settings.':  # noqa
    '',

    '## Basic usage':
    '',

    'You can specify the key and value\'s type at the `Dictionary` type-annotation, as follows:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\ndict_value: ap.Dictionary[str, int] = ap.Dictionary({\'a\': 10})\na_value: int = dict_value[\'a\']\n```':  # noqa
    '',

    'These generic type-annotations are sometimes helpful for checking with the mypy, Pylance, or other libraries and enhancing safety.\n\nFor example, the following code raises an error of value\'s type when checking with the Pylance:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\ndict_value: ap.Dictionary[str, int] = ap.Dictionary({\'a\': 10})\na_value: str = dict_value[\'a\']\n```':  # noqa
    '',

    '':
    '',

    '```\nExpression of type "int" cannot be assigned to declared type "str"\n  "int" is incompatible with "str"\n```':  # noqa
    '',

    'Also, the following code raises an error of key\'s type (`str` is required but `int` is specified):':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\ndict_value: ap.Dictionary[str, int] = ap.Dictionary({\'a\': 10})\na_value: int = dict_value[10]\n```':  # noqa
    '',

    'If you need to use multiple types and type checking, then use the `Union`\\, as follows:\n\nNotes: Alternatively, use the `|` symbol, if you are using Python 3.10 or later) or `Any` type.':  # noqa
    '',

    '```py\n# runnable\nfrom typing import Union\n\nimport apysc as ap\n\n# Accepting the str and int key types.\ndict_value: ap.Dictionary[Union[int, str], int] = ap.Dictionary(\n    {\'a\': 10, 2: 20})\na_value: int = dict_value[\'a\']\nb_value: int = dict_value[2]\n```':  # noqa
    '',

    '':
    '',

    '```py\n# runnable\nfrom typing import Any\n\nimport apysc as ap\n\n# Accepting all types by specifying the Any type.\ndict_value: ap.Dictionary[Any, Any] = ap.Dictionary(\n    {\'a\': 10, 2: \'b\'})\na_value: int = dict_value[\'a\']\nb_value: str = dict_value[2]\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: animation_fill_alpha.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# animation_fill_alpha interface':
    '',

    'This page explains the `animation_fill_alpha` method interface.':
    '',

    '## What interface is this?':
    '',

    'The `animation_fill_alpha` method interface creates an `ap.AnimationFillAlpha` instance (animation setting instance and the  `AnimationBase` subclass). You can animate fill alpha (opacity) with it.\n\nThis interface exists on a `GraphicsBase` subclass, such as the `Rectangle` or `Circle` class.':  # noqa
    '',

    '## Basic usage':
    '',

    'The following example sets the fill alpha animation (from 1.0 to 0.0) with the `animation_fill_alpha` method:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nDURATION: int = 1000\n\n\ndef on_animation_complete_1(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    rectangle.animation_fill_alpha(\n        alpha=1.0, duration=DURATION,\n    ).animation_complete(on_animation_complete_2).start()\n\n\ndef on_animation_complete_2(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    rectangle.animation_fill_alpha(\n        alpha=0.0, duration=DURATION,\n    ).animation_complete(on_animation_complete_1).start()\n\n\nap.Stage(\n    stage_width=150, stage_height=150,\n    background_color=\'#333\', stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#00aaff\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle.animation_fill_alpha(\n    alpha=0.0, duration=DURATION,\n).animation_complete(on_animation_complete_1).start()\n\nap.save_overall_html(\n    dest_dir_path=\'./animation_fill_alpha_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/animation_fill_alpha_basic_usage/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## animation_fill_alpha API':
    '',

    '<!-- Docstring: apysc._animation.animation_fill_alpha_interface.AnimationFillAlphaInterface.animation_fill_alpha -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `animation_fill_alpha(self, alpha:Union[float, apysc._type.number.Number], *, duration:Union[int, apysc._type.int.Int]=3000, delay:Union[int, apysc._type.int.Int]=0, easing:apysc._animation.easing.Easing=<Easing.LINEAR: \'function(x) {return x;}\'>) -> apysc._animation.animation_fill_alpha.AnimationFillAlpha`<hr>\n\n**[Interface summary]** Set the fill alpha (opacity) animation setting.<hr>\n\n**[Parameters]**\n\n- `alpha`: Number or float\n  - The final alpha (opacity) of the animation.\n- `duration`: Int or int, default 3000\n  - Milliseconds before an animation ends.\n- `delay`: Int or int, default 0\n  - Milliseconds before an animation starts.\n- `easing`: Easing, default Easing.LINEAR\n  - Easing setting.\n\n<hr>\n\n**[Returns]**\n\n- `animation_fill_alpha`: AnimationFillAlpha\n  - Created animation setting instance.\n\n<hr>\n\n**[Notes]**\n\nTo start this animation, you need to call the `start` method of the returned instance.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> circle: ap.Circle = sprite.graphics.draw_circle(\n...     x=100, y=100, radius=50)\n>>> _ = circle.animation_y(\n...     y=100,\n...     duration=1500,\n...     easing=ap.Easing.EASE_OUT_QUINT,\n... ).start()\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [Animation interfaces duration setting document](https://simon-ritchie.github.io/apysc/animation_duration.html)\n- [Each animation interface return value document](https://simon-ritchie.github.io/apysc/animation_return_value.html)\n- [Sequential animation setting document](https://simon-ritchie.github.io/apysc/sequential_animation.html)\n- [animation_parallel interface document](https://simon-ritchie.github.io/apysc/animation_parallel.html)\n- [Easing enum document](https://simon-ritchie.github.io/apysc/easing_enum.html)':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: display_object_parent.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Display object parent interfaces':
    '',

    'This page explains the `DisplayObject` class `parent` interfaces (the `parent` property and `remove_from_parent` method).':  # noqa
    '',

    '## What interfaces are these?':
    '',

    'The `parent` attribute is the getter property. This attribute becomes a `Stage` instance or a container instance like a `Sprite` instance.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=200,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\nassert isinstance(sprite.parent, ap.Stage)\nassert isinstance(sprite.graphics.parent, ap.Sprite)\nassert isinstance(rectangle.parent, ap.Graphics)\n```':  # noqa
    '',

    'Notes: This interface automatically adds the sprite to the stage at the constructor. Similarly, this interface adds the `Graphics` instance to the `Sprite` instance.\n\nThe `remove_from_parent` interface removes self-instance from the parent (and not be displayed on the stage).':  # noqa
    '',

    '## Basic usage of the remove_from_parent interface':
    '',

    'The `remove_from_parent` method interface (no argument options) removes the self-instance from the parent. A Removed instance is not displayed until any parent adds it again.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\n# Remove the rectangle from the parent, and nothing displays\n# on the stage.\nrectangle.remove_from_parent()\n\nap.save_overall_html(\n    dest_dir_path=\'display_object_remove_from_parent_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/display_object_remove_from_parent_basic_usage/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## See also':
    '',

    '- [Add child and remove child interfaces](add_child_and_remove_child.md)':  # noqa
    '',

    '## parent API':
    '',

    '<!-- Docstring: apysc._display.parent_interface.ParentInterface.parent -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get parent instance that has a add_child and remove_child interfaces.<hr>\n\n**[Returns]**\n\n- `parent`: any parent instance or None\n  - Parent instance with `add_child` and `remove_child` interfaces. If this instance does not have a parent instance (not added child), this interface returns None.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite_1: ap.Sprite = ap.Sprite()\n>>> sprite_1.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite_1.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> sprite_2: ap.Sprite = ap.Sprite()\n>>> sprite_2.add_child(rectangle)\n>>> rectangle.parent == sprite_2\nTrue\n```':  # noqa
    '',

}



"""Class implementation for the mouse event.
"""

from typing import Generic
from typing import TypeVar

from apysc._event.event import Event
from apysc._event.prevent_default_interface import PreventDefaultInterface
from apysc._event.stop_propagation_interface import StopPropagationInterface
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.int import Int
from apysc._type.variable_name_interface import VariableNameInterface

T = TypeVar('T', bound=VariableNameInterface)


class MouseEvent(
        Event[T], Generic[T], StopPropagationInterface,
        PreventDefaultInterface):
    """
    Mouse event class.

    References
    ----------
    - Basic mouse event interfaces
        - https://simon-ritchie.github.io/apysc/mouse_event_basic.html

    Examples
    --------
    >>> import apysc as ap
    >>> def on_mousedown(
    ...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:
    ...     rectangle: ap.Rectangle = e.this
    ...     rectangle.fill_color = ap.String('#f0a')
    ...     rectangle.unbind_mousedown_all()
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.begin_fill(color='#0af')
    >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
    ...     x=50, y=50, width=50, height=50)
    >>> _ = rectangle.mousedown(on_mousedown)
    """

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='MouseEvent')
    def __init__(self, this: T) -> None:
        """
        Mouse event class.

        Parameters
        ----------
        this : VariableNameInterface
            Instance that listening event (e.g., Sprite).

        References
        ----------
        - Basic mouse event interfaces
            - https://simon-ritchie.github.io/apysc/mouse_event_basic.html

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> e: ap.MouseEvent = ap.MouseEvent(this=rectangle)
        """
        from apysc._expression import var_names
        super(MouseEvent, self).__init__(
            this=this, type_name=var_names.MOUSE_EVENT)

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='MouseEvent')
    def stage_x(self) -> Int:
        """
        Get the x-coordinate of the stage reference.

        Returns
        -------
        x : Int
            x-coordinate.

        References
        ----------
        - Common mouse event interfaces document
            - https://simon-ritchie.github.io/apysc/mouse_event_basic.html

        Examples
        --------
        >>> import apysc as ap
        >>> def on_mousedown(
        ...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:
        ...     stage_x: ap.Int = e.stage_x
        ...     # Do something here with the coordinate.
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> _ = rectangle.mousedown(on_mousedown)
        """
        import apysc as ap
        x: ap.Int = ap.Int(0)
        self._append_stage_x_getter_expression(x=x)
        return x

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='MouseEvent')
    def _append_stage_x_getter_expression(self, *, x: Int) -> None:
        """
        Append stage_x getter property expression.

        Parameters
        ----------
        x : Int
            Target x-coordinate value.
        """
        import apysc as ap
        from apysc._display.stage import get_stage_elem_str
        expression: str = (
            f'{x.variable_name} = {self.variable_name}.pageX - '
            f'{get_stage_elem_str()}.offset().left;'
        )
        ap.append_js_expression(expression=expression)

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='MouseEvent')
    def stage_y(self) -> Int:
        """
        Get the y-coordinate of the stage reference.

        Returns
        -------
        y : Int
            y-coordinate.

        References
        ----------
        - Common mouse event interfaces document
            - https://simon-ritchie.github.io/apysc/mouse_event_basic.html

        Examples
        --------
        >>> import apysc as ap
        >>> def on_mousedown(
        ...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:
        ...     stage_y: ap.Int = e.stage_y
        ...     # Do something here with the coordinate.
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> _ = rectangle.mousedown(on_mousedown)
        """
        import apysc as ap
        y: ap.Int = ap.Int(0)
        self._append_stage_y_getter_expression(y=y)
        return y

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='MouseEvent')
    def _append_stage_y_getter_expression(self, *, y: Int) -> None:
        """
        Append stage_y getter property expression.

        Parameters
        ----------
        y : Int
            Target y-coordinate value.
        """
        import apysc as ap
        from apysc._display.stage import get_stage_elem_str
        expression: str = (
            f'{y.variable_name} = {self.variable_name}.pageY - '
            f'{get_stage_elem_str()}.offset().top;'
        )
        ap.append_js_expression(expression=expression)

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='MouseEvent')
    def local_x(self) -> Int:
        """
        Get a local x-coordinate event listening instance.
        For example, this value becomes x-coordinate from
        Sprite's left-end position by clicking a Sprite instance.

        Returns
        -------
        x : Int
            x-coordinate.

        References
        ----------
        - Common mouse event interfaces document
            - https://simon-ritchie.github.io/apysc/mouse_event_basic.html

        Examples
        --------
        >>> import apysc as ap
        >>> def on_mousedown(
        ...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:
        ...     local_x: ap.Int = e.local_x
        ...     # Do something here with the coordinate.
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> _ = rectangle.mousedown(on_mousedown)
        """
        import apysc as ap
        x: ap.Int = ap.Int(0)
        self._append_local_x_getter_expression(x=x)
        return x

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='MouseEvent')
    def _append_local_x_getter_expression(self, *, x: Int) -> None:
        """
        Append local_x getter property expression.

        Parameters
        ----------
        x : Int
            Target x-coordinate value.
        """
        import apysc as ap
        stage_x: ap.Int = self.stage_x
        this: T = self.this
        expression: str = (
            f'{x.variable_name} = {stage_x.variable_name} - '
            f'get_total_x({this.variable_name});'
        )
        ap.append_js_expression(expression=expression)

    @property  # type: ignore[misc]
    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='MouseEvent')
    def local_y(self) -> Int:
        """
        Get the local y-coordinate of the event listening instance.
        For example, this value becomes y-coordinate from Sprite's
        top-end position by clicking a Sprite instance.

        Returns
        -------
        y : Int
            y-coordinate.

        References
        ----------
        - Common mouse event interfaces document
            - https://simon-ritchie.github.io/apysc/mouse_event_basic.html

        Examples
        --------
        >>> import apysc as ap
        >>> def on_mousedown(
        ...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:
        ...     local_y: ap.Int = e.local_y
        ...     # Do something here with the coordinate.
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> _ = rectangle.mousedown(on_mousedown)
        """
        import apysc as ap
        y: ap.Int = ap.Int(0)
        self._append_local_y_getter_expression(y=y)
        return y

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='MouseEvent')
    def _append_local_y_getter_expression(self, *, y: Int) -> None:
        """
        Append local_y getter property expression.

        Parameters
        ----------
        y : Int
            Target y-coordinate value.
        """
        import apysc as ap
        stage_y: ap.Int = self.stage_y
        this: T = self.this
        expression: str = (
            f'{y.variable_name} = {stage_y.variable_name} - '
            f'get_total_y({this.variable_name});'
        )
        ap.append_js_expression(expression=expression)



"""Class implementation for the animation_height (for ellipse) interface.
"""

from typing import Union

from apysc._animation.animation_height_for_ellipse import \
    AnimationHeightForEllipse
from apysc._animation.animation_interface_base import AnimationInterfaceBase
from apysc._animation.easing import Easing
from apysc._type.int import Int


class AnimationHeightForEllipseInterface(AnimationInterfaceBase):

    def animation_height(
            self,
            height: Union[int, Int],
            *,
            duration: Union[int, Int] = 3000,
            delay: Union[int, Int] = 0,
            easing: Easing = Easing.LINEAR) -> AnimationHeightForEllipse:
        """
        Set the ellipse-height animation setting.

        Notes
        -----
        To start this animation, you need to call the `start` method of
        the returned instance.

        Parameters
        ----------
        height : Int or int
            The final ellipse-height of the animation.
        duration : Int or int, default 3000
            Milliseconds before an animation ends.
        delay : Int or int, default 0
            Milliseconds before an animation starts.
        easing : Easing, default Easing.LINEAR
            Easing setting.

        Returns
        -------
        animation_height_for_ellipse : AnimationHeightForEllipse
            Created animation setting instance.

        References
        ----------
        - animation_width and animation_height interfaces document
            - https://simon-ritchie.github.io/apysc/animation_width_and_height.html  # noqa
        - Animation interfaces duration setting document
            - https://simon-ritchie.github.io/apysc/animation_duration.html
        - Animation interfaces delay setting document
            - https://simon-ritchie.github.io/apysc/animation_delay.html
        - Each animation interface return value document
            - https://simon-ritchie.github.io/apysc/animation_return_value.html  # noqa
        - Sequential animation setting document
            - https://simon-ritchie.github.io/apysc/sequential_animation.html
        - animation_parallel interface document
            - https://simon-ritchie.github.io/apysc/animation_parallel.html
        - Easing enum document
            - https://simon-ritchie.github.io/apysc/easing_enum.html

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> ellipse: ap.Ellipse = sprite.graphics.draw_ellipse(
        ...     x=50, y=50, width=50, height=50)
        >>> _ = ellipse.animation_height(
        ...     height=100,
        ...     duration=1500,
        ...     easing=ap.Easing.EASE_OUT_QUINT,
        ... ).start()
        """
        animation_height_for_ellipse: AnimationHeightForEllipse = \
            AnimationHeightForEllipse(
                target=self,
                height=height,
                duration=duration,
                delay=delay,
                easing=easing)
        return animation_height_for_ellipse



"""This module is for the translation mapping data of the
following document:

Document file: animation_event.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# AnimationEvent':
    '',

    'This page explains the `AnimationEvent` class.':
    '',

    '## What class is this?':
    '',

    'An animation-related event handler uses the `AnimationEvent` class, such as the complete animation event. Each animation interface passes this event instance to the handler.':  # noqa
    '',

    '## Basic usage':
    '',

    'The following example sets the animation complete event handler. The animation interface passes the `AnimationEvent` instance argument as the `e: ap.AnimationEvent`:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_animation_complete(e: ap.AnimationEvent, options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    ap.trace(\'Animation is completed!\')\n\n\nap.Stage(\n    stage_width=200, stage_height=150,\n    background_color=\'#333\', stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#00aaff\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nanimation_x: ap.AnimationX = rectangle.animation_x(x=100)\nanimation_x.animation_complete(on_animation_complete)\nanimation_x.start()\n```':  # noqa
    '',

    '':
    '',

    '## this property':
    '',

    'The `AnimationEvent` instance\'s `this` property is a subclass instance of the `AnimationBase` class, such as the `AnimationMove`\\, `AnimationX`\\, or other class.\n\nThis type depends on the called interface, e.g., if you use the `animation_x` interface, `this` property type becomes an `AnimationX` instance.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_animation_complete(e: ap.AnimationEvent, options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    assert isinstance(e.this, ap.AnimationX)\n\n\nap.Stage(\n    stage_width=200, stage_height=150,\n    background_color=\'#333\', stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#00aaff\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nanimation_x: ap.AnimationX = rectangle.animation_x(x=100)\nanimation_x.animation_complete(on_animation_complete)\nanimation_x.start()\n```':  # noqa
    '',

    '':
    '',

    '## Generic type annotation':
    '',

    'The `AnimationEvent` class can set a generic type annotation. If you set a generic type annotation, then the animation target property type (e.g., `DisplayObject`) changes.\n\nThe following example sets a `Rectangle` generic type annotation to the `AnimationEvent` and benefits from type checking libraries (such as the `mypy` or `Pylance`).':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_animation_complete(\n        e: ap.AnimationEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that animation calls when its end.\n\n    Parameters\n    ----------\n    e : ap.AnimationEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this.target\n    rectangle.animation_x(x=50).start()\n\n\nap.Stage(\n    stage_width=200, stage_height=150,\n    background_color=\'#333\', stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#00aaff\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nanimation_x: ap.AnimationX = rectangle.animation_x(x=100)\nanimation_x.animation_complete(on_animation_complete)\nanimation_x.start()\n```':  # noqa
    '',

    '':
    '',

    '## AnimationEvent constructor API':
    '',

    '<!-- Docstring: apysc._event.animation_event.AnimationEvent.__init__ -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `__init__(self, *, this:\'animation_base.AnimationBase[_T]\') -> None`<hr>\n\n**[Interface summary]** Animation event class.<hr>\n\n**[Parameters]**\n\n- `this`: AnimationBase\n  - Animation setting instance.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_animation_complete(\n...         e: ap.AnimationEvent[ap.Rectangle],\n...         options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this.target\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.animation_x(\n...     x=100).animation_complete(on_animation_complete)\n```':  # noqa
    '',

    '':
    '',

    '## this property API':
    '',

    '<!-- Docstring: apysc._event.animation_event.AnimationEvent.this -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get an animation setting instance of listening to this event.<hr>\n\n**[Returns]**\n\n- `this`: AnimationBase\n  - Instance of listening to this event.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_animation_complete(\n...         e: ap.AnimationEvent[ap.Rectangle],\n...         options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this.target\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.animation_x(\n...     x=100).animation_complete(on_animation_complete)\n```':  # noqa
    '',

}



"""Implementations of Polygon class.
"""


from apysc._display import graphics
from apysc._display.append_line_point_interface import AppendLinePointInterface
from apysc._display.line_base import LineBase
from apysc._geom.point2d import Point2D
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.array import Array


class Polygon(LineBase, AppendLinePointInterface):
    """
    The polygon vector graphics class.

    References
    ----------
    - Graphics draw_polygon interface document
        - https://simon-ritchie.github.io/apysc/graphics_draw_polygon.html  # noqa

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite: ap.Sprite = ap.Sprite()
    >>> sprite.graphics.begin_fill(color='#0af')
    >>> polygon: ap.Polygon = sprite.graphics.draw_polygon(
    ...     points=[
    ...         ap.Point2D(x=50, y=50),
    ...         ap.Point2D(x=50, y=100),
    ...         ap.Point2D(x=100, y=75),
    ...     ])
    >>> polygon.fill_color
    String('#00aaff')
    """

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='Polygon')
    def __init__(
            self, *, parent: 'graphics.Graphics',
            points: Array[Point2D]) -> None:
        """
        Create a polygon vector graphic. This class is
        similar to the Polyline class, but unlike that,
        this class connects an end-point and start-point.

        Parameters
        ----------
        parent : Graphics
            Graphics instance to link this graphic.
        points : Array of Point2D
            List of polygon vertex points.

        References
        ----------
        - Graphics draw_polygon interface document
            - https://simon-ritchie.github.io/apysc/graphics_draw_polygon.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> polygon: ap.Polygon = sprite.graphics.draw_polygon(
        ...     points=[
        ...         ap.Point2D(x=50, y=50),
        ...         ap.Point2D(x=50, y=100),
        ...         ap.Point2D(x=100, y=75),
        ...     ])
        >>> polygon.fill_color
        String('#00aaff')
        """
        from apysc._display.graphics import Graphics
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        parent_graphics: Graphics = parent
        variable_name: str = expression_variables_util.\
            get_next_variable_name(type_name=var_names.POLYGON)
        super(Polygon, self).__init__(
            parent=parent, x=0, y=0, variable_name=variable_name)
        self.points = points
        self._set_initial_basic_values(parent=parent)
        self._append_constructor_expression()
        self._set_line_setting_if_not_none_value_exists(
            parent_graphics=parent_graphics)
        self._set_overflow_visible_setting()

    def __repr__(self) -> str:
        """
        Get a string representation of this instance (for the sake of
        debugging).

        Returns
        -------
        repr_str : str
            Type name and variable name will be set
            (e.g., `Polygon('<variable_name>')`).
        """
        repr_str: str = f"Polygon('{self.variable_name}')"
        return repr_str

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='Polygon')
    def _append_constructor_expression(self) -> None:
        """
        Append constructor expression.
        """
        import apysc as ap
        from apysc._display.stage import get_stage_variable_name
        stage_variable_name: str = get_stage_variable_name()
        points_var_name: str
        points_expression: str
        points_var_name, points_expression = \
            self._make_2dim_points_expression()
        expression: str = (
            f'{points_expression}'
            f'\nvar {self.variable_name} = {stage_variable_name}'
            f'\n  .polygon({points_var_name})'
            '\n  .attr({'
        )
        expression = self._append_basic_vals_expression(
            expression=expression, indent_num=2)
        expression += '\n  });'
        ap.append_js_expression(expression=expression)
        self._points_var_name = points_var_name



"""This module is for the translation mapping data of the
following document:

Document file: what_apysc_can_do.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# What apysc can do in its current implementation':
    '',

    'This page explains the current apysc implementation and functionality summary.':  # noqa
    '',

    '## Write with the Python and export HTML or use it on the Jupyter':
    '',

    'The apysc library can write the front-end with the Python language and export the HTML or use it on the Jupyter notebook, JupyterLab, and the Google Colaboratory!\n\nSee also:\n\n- [Save overall html interface](save_overall_html.md)\n- [Display on the jupyter interface](display_on_jupyter.md)\n- [Display on the Google Colaboratory interface](display_on_colaboratory.md)':  # noqa
    '',

    '## Draw the many types of the vector graphics':
    '',

    'The apysc library can draw many vector graphics types, like the rectangle, circle, line.\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=650, stage_height=210, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=\'#0af\')\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nsprite.graphics.draw_round_rect(\n    x=150, y=50, width=50, height=50, ellipse_width=12, ellipse_height=12)\n\nsprite.graphics.draw_circle(x=275, y=75, radius=25)\n\nsprite.graphics.draw_ellipse(x=375, y=75, width=50, height=30)\n\nsprite.graphics.draw_polygon(\n    points=[\n        ap.Point2D(x=475, y=50),\n        ap.Point2D(x=450, y=100),\n        ap.Point2D(x=500, y=100),\n    ])\n\nsprite.graphics.begin_fill(color=\'\')\nsprite.graphics.line_style(color=\'#eee\', thickness=3)\nsprite.graphics.move_to(x=550, y=50)\nsprite.graphics.line_to(x=600, y=50)\nsprite.graphics.line_to(x=550, y=100)\nsprite.graphics.line_to(x=600, y=100)\n\nsprite.graphics.draw_line(x_start=50, y_start=130, x_end=600, y_end=130)\nsprite.graphics.draw_dotted_line(\n    x_start=50, y_start=130, x_end=600, y_end=130, dot_size=5)\nsprite.graphics.draw_round_dotted_line(\n    x_start=53, y_start=160, x_end=600, y_end=160, round_size=6, space_size=6)\n\nap.save_overall_html(\n    dest_dir_path=\'what_apysc_can_do_draw_vector_graphics/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/what_apysc_can_do_draw_vector_graphics/index.html" width="650" height="210"></iframe>\n\nSee also:\n\n- [Graphics class](graphics.md)\n- [Graphics class begin fill interface](graphics_begin_fill.md)\n- [Graphics class line style interface](graphics_line_style.md)\n- [Graphics class draw rect interface](graphics_draw_rect.md)\n- [Graphics class draw round rect interface](graphics_draw_round_rect.md)\n- [Graphics class draw circle interface](graphics_draw_circle.md)\n- [Graphics class draw ellipse interface](graphics_draw_ellipse.md)\n- [Graphics class move to and line to interfaces](graphics_move_to_and_line_to.md)\n- [Graphics class draw line interface](graphics_draw_line.md)\n- [Graphics class draw dotted line interface](graphics_draw_dotted_line.md)\n- [Graphics class draw dashed line interface](graphics_draw_dashed_line.md)\n- [Graphics class draw round dotted line interface](graphics_draw_round_dotted_line.md)\n- [Graphics class draw dash dotted line interface](graphics_draw_dash_dotted_line.md)\n- [Graphics class draw polygon interface](graphics_draw_polygon.md)':  # noqa
    '',

    '## Set each mouse event':
    '',

    'The apysc library supports each mouse event, like the click, mouse down, mouse over, mouse move.\n\nThe click event example (please click the following rectangle):\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_click(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when clicked.\n\n    Parameters\n    ----------\n    e : ap.MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    color: ap.String = e.this.fill_color\n    condition: ap.Boolean = color == \'#00aaff\'\n    with ap.If(condition):\n        e.this.fill_color = ap.String(\'#f0a\')\n    with ap.Else():\n        e.this.fill_color = ap.String(\'#0af\')\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\nrectangle.click(on_click)\n\nap.save_overall_html(\n    dest_dir_path=\'what_apysc_can_do_mouse_event_click/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/what_apysc_can_do_mouse_event_click/index.html" width="150" height="150"></iframe>\n\nSee also:\n\n- [Basic mouse event interfaces](mouse_event_basic.md)\n- [Click interface](click.md)\n- [Mousedown and mouseup interfaces](mousedown_and_mouseup.md)\n- [Mouseover and mouseout interfaces](mouseover_and_mouseout.md)\n- [Mousemove interface](mousemove.md)':  # noqa
    '',

    '## The timer interface and animation':
    '',

    'You can use the timer-related interfaces and animate with those.\n\n<details>\n<summary>Display the code block:</summary>':  # noqa
    '',

    '```py\n# runnable\nfrom typing_extensions import TypedDict\n\nimport apysc as ap\n\n\nclass _Options(TypedDict):\n    rectangle: ap.Rectangle\n    alpha_direction: ap.Int\n\n\ndef on_timer(e: ap.TimerEvent, options: _Options) -> None:\n    """\n    The handler that the timer calls.\n\n    Parameters\n    ----------\n    e : ap.TimerEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = options[\'rectangle\']\n    alpha_direction: ap.Int = options[\'alpha_direction\']\n    current_alpha: ap.Number = rectangle.fill_alpha\n    condition_1: ap.Boolean = current_alpha < 0.0\n    condition_2: ap.Boolean = current_alpha > 1.0\n    with ap.If(condition_1):\n        alpha_direction.value = 1\n    with ap.Elif(condition_2):\n        alpha_direction.value = -1\n    rectangle.fill_alpha += alpha_direction * 0.03\n    rectangle.rotation_around_center += 1\n\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\nsprite.graphics.begin_fill(color=\'#0af\')\nalpha_direction: ap.Int = ap.Int(1)\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\noptions: _Options = {\n    \'rectangle\': rectangle, \'alpha_direction\': alpha_direction}\ntimer: ap.Timer = ap.Timer(\n    on_timer, delay=ap.FPS.FPS_60, options=options)\ntimer.start()\n\nap.save_overall_html(\n    dest_dir_path=\'what_apysc_can_do_timer_animation/\')\n```':  # noqa
    '',

    '</details>\n\n<iframe src="static/what_apysc_can_do_timer_animation/index.html" width="150" height="150"></iframe>\n\nSee also:\n\n- [Timer class](timer.md)\n- [TimerEvent class](timer_event.md)\n- [Timer class delay setting](timer_delay.md)\n- [FPS enum](fps.md)\n- [Timer class repeat count setting](timer_repeat_count.md)\n- [Timer class start and stop interfaces](timer_start_and_stop.md)\n- [Timer class timer complete interface](timer_complete.md)\n- [Timer class reset interface](timer_reset.md)':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: array_reverse.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Array class reverse interface':
    '',

    'This page explains the `Array` class `reverse` method interface.':
    '',

    '## What interface is this?':
    '',

    'The `reverse` method interface reverses an array\'s values order in place.':  # noqa
    '',

    '## Basic usage':
    '',

    'The `reverse` method requires no arguments and returns no value.':
    '',

    '```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 3, 5])\narr.reverse()\nassert arr == [5, 3, 1]\n```':  # noqa
    '',

    '':
    '',

    '## See also':
    '',

    '- [Array class sort interface](array_sort.md)':
    '',

    '## reverse API':
    '',

    '<!-- Docstring: apysc._type.array.Array.reverse -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `reverse(self) -> None`<hr>\n\n**[Interface summary]** Reverse this array in place.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr.reverse()\n>>> arr\nArray([3, 2, 1])\n```':  # noqa
    '',

}



"""Each interface to get a Python built-in value from an apysc one.
"""

from typing import Union

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.string import String


@add_debug_info_setting(  # type: ignore[misc]
    module_name=__name__)
def get_builtin_str_from_apysc_val(*, string: Union[str, String]) -> str:
    """
    Get a Python built-in string from an apysc one.

    Parameters
    ----------
    string : str or ap.String
        Target string value.

    Returns
    -------
    builtin_val : str
        A Python built-in string.
    """
    if isinstance(string, str):
        return string
    builtin_val: str = string._value
    return builtin_val



"""This module is for the translation mapping data of the
following document:

Document file: mousedown_and_mouseup.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Mousedown and mouseup interfaces':
    '',

    'This page explains the `mousedown` and `mouseup` interfaces.':
    '',

    '## What interfaces are these?':
    '',

    'The `mousedown` interface binds the event handler that the interface calls when a user mouse downed on a `DisplayObject` instance. Conversely, the `mouseup` interface binds the event handler that the interface calls when a user mouse upped on a `DisplayObject` one.':  # noqa
    '',

    '## See also':
    '',

    'The following page describes the basic mouse event interfaces:\n\n- [Basic mouse event interfaces](mouse_event_basic.md)':  # noqa
    '',

    '## Basic usage of the mousedown and mouseup interfaces':
    '',

    'Each `DisplayObject` instance has the `mousedown` and `mouseup` method interfaces, and you can bind handlers by these.\n\nThe following example binds the mouse down handler and mouse upped one to the rectangle. The handler changes the rectangle color when the mouse downs and reverts to the original one when the mouse upped.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_mousedown(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mousedown.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.String(\'#f0a\')\n\n\ndef on_mouseup(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mouseup.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.fill_color = ap.String(\'#0af\')\n\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\n# Bind each handler to the rectangle.\nrectangle.mousedown(on_mousedown)\nrectangle.mouseup(on_mouseup)\n\nap.save_overall_html(\n    dest_dir_path=\'mousedown_and_mouseup_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/mousedown_and_mouseup_basic_usage/index.html" width="150" height="150"></iframe>':  # noqa
    '',

    '## Unbind interfaces':
    '',

    'The `unbind_mousedown` and `unbind_mouseup` interfaces unbind each registered handler from the `DisplayObject`\\.\n\nThe following example unbinds handlers in the `on_mousedown` and `on_mouseup` functions so that the rectangle calls these handlers only once.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\n\ndef on_mousedown(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mousedown.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.unbind_mousedown(handler=on_mousedown)\n    rectangle.fill_color = ap.String(\'#f0a\')\n\n\ndef on_mouseup(\n        e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n    """\n    The handler that the rectangle calls when mouseup.\n\n    Parameters\n    ----------\n    e : MouseEvent\n        Event instance.\n    options : dict\n        Optional arguments dictionary.\n    """\n    rectangle: ap.Rectangle = e.this\n    rectangle.unbind_mouseup(handler=on_mouseup)\n    rectangle.fill_color = ap.String(\'#0af\')\n\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=150,\n    stage_height=150,\n    stage_elem_id=\'stage\')\n\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nrectangle: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\nrectangle.mousedown(on_mousedown)\nrectangle.mouseup(on_mouseup)\n\nap.save_overall_html(\n    dest_dir_path=\'mousedown_and_mouseup_unbind_interfaces/\')\n```':  # noqa
    '',

    '<iframe src="static/mousedown_and_mouseup_unbind_interfaces/index.html" width="150" height="150"></iframe>\n\nThere are also existing the `unbind_mousedown_all` and `unbind_mouseup_all` interfaces. These interfaces unbind all the handlers from the target `DisplayObject` instance.':  # noqa
    '',

    '## mousedown API':
    '',

    '<!-- Docstring: apysc._event.mouse_down_interface.MouseDownInterface.mousedown -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `mousedown(self, handler:Callable[[apysc._event.mouse_event.MouseEvent, ~_O], NoneType], *, options:Union[~_O, NoneType]=None) -> str`<hr>\n\n**[Interface summary]** Add mouse down event listener setting.<hr>\n\n**[Parameters]**\n\n- `handler`: _Handler\n  - Callable that would be called when mouse down on this instance.\n- `options`: dict or None, default None\n  - Optional arguments dictionary to be passed to a handler.\n\n<hr>\n\n**[Returns]**\n\n- `name`: str\n  - Handler\'s name.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mousedown(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mousedown(on_mousedown)\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [About the handler options’ type document](https://simon-ritchie.github.io/apysc/about_handler_options_type.html)':  # noqa
    '',

    '## unbind_mousedown API':
    '',

    '<!-- Docstring: apysc._event.mouse_down_interface.MouseDownInterface.unbind_mousedown -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `unbind_mousedown(self, handler:Callable[[apysc._event.mouse_event.MouseEvent, ~_O], NoneType]) -> None`<hr>\n\n**[Interface summary]** Unbind a specified handler\'s mouse down event.<hr>\n\n**[Parameters]**\n\n- `handler`: _Handler\n  - Unbinding target Callable.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mousedown(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n...     rectangle.unbind_mousedown(on_mousedown)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mousedown(on_mousedown)\n```':  # noqa
    '',

    '':
    '',

    '## unbind_mousedown_all API':
    '',

    '<!-- Docstring: apysc._event.mouse_down_interface.MouseDownInterface.unbind_mousedown_all -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `unbind_mousedown_all(self) -> None`<hr>\n\n**[Interface summary]** Unbind all mouse down events.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mousedown(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n...     rectangle.unbind_mousedown_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mousedown(on_mousedown)\n```':  # noqa
    '',

    '':
    '',

    '## mouseup API':
    '',

    '<!-- Docstring: apysc._event.mouse_up_interface.MouseUpInterface.mouseup -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `mouseup(self, handler:Callable[[apysc._event.mouse_event.MouseEvent, ~_O], NoneType], *, options:Union[~_O, NoneType]=None) -> str`<hr>\n\n**[Interface summary]** Add mouse up event listener setting.<hr>\n\n**[Parameters]**\n\n- `handler`: _Handler\n  - Callable that would be called when mouse-up on this instance.\n- `options`: dict or None, default None\n  - Optional arguments dictionary to be passed to a handler\n\n<hr>\n\n**[Returns]**\n\n- `name`: str\n  - Handler\'s name.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mouseup(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mouseup(on_mouseup)\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [About the handler options’ type document](https://simon-ritchie.github.io/apysc/about_handler_options_type.html)':  # noqa
    '',

    '## unbind_mouseup API':
    '',

    '<!-- Docstring: apysc._event.mouse_up_interface.MouseUpInterface.unbind_mouseup -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `unbind_mouseup(self, handler:Callable[[apysc._event.mouse_event.MouseEvent, ~_O], NoneType]) -> None`<hr>\n\n**[Interface summary]** Unbind a specified handler\'s mouse-up event.<hr>\n\n**[Parameters]**\n\n- `handler`: _Handler\n  - Unbinding target Callable.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mouseup(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n...     rectangle.unbind_mouseup(on_mouseup)\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mouseup(on_mouseup)\n```':  # noqa
    '',

    '':
    '',

    '## unbind_mouseup_all API':
    '',

    '<!-- Docstring: apysc._event.mouse_up_interface.MouseUpInterface.unbind_mouseup_all -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `unbind_mouseup_all(self) -> None`<hr>\n\n**[Interface summary]** Unbind all mouse up events.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> def on_mouseup(\n...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:\n...     rectangle: ap.Rectangle = e.this\n...     rectangle.fill_color = ap.String(\'#f0a\')\n...     rectangle.unbind_mouseup_all()\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> _ = rectangle.mouseup(on_mouseup)\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: display_on_jupyter.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# display_on_jupyter interface':
    '',

    'This page explains the `display_on_jupyter` function interface.':
    '',

    '## What interface is this?':
    '',

    'The `display_on_jupyter` interface displays the apysc HTML on the Jupyter.':  # noqa
    '',

    '## Requirements':
    '',

    'This interface requires the Jupyter library. Therefore, if you haven\'t installed Jupyter, you need to install it before going on (e.g., `pip install notebook`).\n\nFor more information, please see:\n\n- [Installing the Jupyter Software](https://jupyter.org/install)\n\nAlso, this interface uses the `IPython.display.IFrame` interface. If you encountered that interface error, please update the Jupyter version.':  # noqa
    '',

    '## Notes':
    '',

    '- Jupyter on the VS Code is not supported currently (since the VS code restriction).\n- Jupyter notebook and JupyterLab are supported.':  # noqa
    '',

    '## Basic usage':
    '',

    'You can use the `display_on_jupyter` interface to display an output HTML instead of the `save_overall_html` function.\n\nThis interface requires the `html_file_name` argument to be unique if you need to output multiple HTML. Otherwise, this interface overwrites the HTML file:':  # noqa
    '',

    '```py\nimport apysc as ap\n\nap.Stage(\n    stage_width=250, stage_height=150, background_color=\'#333\')\nsprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\nsprite.graphics.draw_rect(x=50, y=50, width=50, height=50)\n\nsprite.graphics.begin_fill(color=\'#f0a\')\nsprite.graphics.draw_rect(x=150, y=50, width=50, height=50)\n\nap.display_on_jupyter(html_file_name=\'jupyter_sample_1.html\')\n```':  # noqa
    '',

    '![](_static/jupyter_notebook_interface.png)\n\nAlso, this interface can use on the JupyterLab:\n\n![](_static/jupyterlab_interface.png)':  # noqa
    '',

    '## display_on_jupyter API':
    '',

    '<!-- Docstring: apysc._jupyter.jupyter_util.display_on_jupyter -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `display_on_jupyter(html_file_name:str, *, minify:bool=True) -> None`<hr>\n\n**[Interface summary]** Save the overall HTML and display it on the Jupyter.<hr>\n\n**[Parameters]**\n\n- `html_file_name`: str, default \'index.html\'\n  - The output HTML file name.\n- `minify`: bool, default True\n  - Boolean value whether minify a HTML or not. False setting is useful when debugging.\n\n<hr>\n\n**[Notes]**\n\nCurrently, this interface does not support Jupyter on the VS Code. This interface requires the Jupyter library (e.g., `notebook` package).':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: array_index_of.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Array class index_of interface':
    '',

    'This page explains the `Array` class `index_of` method interface.':
    '',

    '## What interface is this?':
    '',

    'The `index_of` method returns the specified value\'s index in the array.':  # noqa
    '',

    '## Basic usage':
    '',

    'The `index_of` method requires the `value` argument and returns the found value\'s index in the array.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 3, 5])\nindex: ap.Int = arr.index_of(value=3)\nassert index == 1\n```':  # noqa
    '',

    'If there is no found value, the return index becomes `-1`.':
    '',

    '```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 3, 5])\nindex: ap.Int = arr.index_of(value=2)\nassert index == -1\n```':  # noqa
    '',

    '':
    '',

    '## index_of API':
    '',

    '<!-- Docstring: apysc._type.array.Array.index_of -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `index_of(self, value:~T) -> apysc._type.int.Int`<hr>\n\n**[Interface summary]** Search specified value\'s index and return it.<hr>\n\n**[Parameters]**\n\n- `value`: *\n  - Any value to search.\n\n<hr>\n\n**[Returns]**\n\n- `index`: Int\n  - Found position of index. If this array does not contain a value, this interface returns -1.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 3, 5])\n>>> arr.index_of(3)\nInt(1)\n```':  # noqa
    '',

}



"""Class implementation for copy interface.
"""

from copy import deepcopy
from typing import Any

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.type_name_interface import TypeNameInterface
from apysc._type.variable_name_interface import VariableNameInterface


class CopyInterface(TypeNameInterface, VariableNameInterface):

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='CopyInterface')
    def _copy(self) -> Any:
        """
        Make a deep copy of this instance.

        Returns
        -------
        result : *
            Copied instance.
        """
        from apysc._expression import expression_variables_util
        from apysc._expression.event_handler_scope import \
            TemporaryNotHandlerScope
        with TemporaryNotHandlerScope():
            result: CopyInterface = deepcopy(self)
            result.variable_name = \
                expression_variables_util.get_next_variable_name(
                    type_name=self.type_name)
            self._append_copy_expression(
                result_variable_name=result.variable_name)
        self._append_value_updating_cpy_exp_to_handler_scope(
            result_variable_name=result.variable_name)
        return result

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='CopyInterface')
    def _append_value_updating_cpy_exp_to_handler_scope(
            self, *, result_variable_name: str) -> None:
        """
        Append a value updating copy expression
        if the current scope is an event handler's one.

        Parameters
        ----------
        result_variable_name : str
            Copied value's variable name.
        """
        import apysc as ap
        from apysc._expression import event_handler_scope
        from apysc._type.type_util import is_immutable_type
        evt_handler_scope_count: int = event_handler_scope.\
            get_current_event_handler_scope_count()
        if evt_handler_scope_count == 0:
            return
        if is_immutable_type(value=self):
            expression: str = (
                f'{result_variable_name} = {self.variable_name};'
            )
        else:
            expression = (
                f'{result_variable_name} = '
                f'cpy({self.variable_name});'
            )
        ap.append_js_expression(expression=expression)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='CopyInterface')
    def _append_copy_expression(self, *, result_variable_name: str) -> None:
        """
        Append copy expression.

        Parameters
        ----------
        result_variable_name : str
            Copied value's variable name.
        """
        import apysc as ap
        from apysc._type.type_util import is_immutable_type
        if is_immutable_type(value=self):
            expression: str = (
                f'var {result_variable_name} = '
                f'{self.variable_name};'
            )
        else:
            expression = (
                f'var {result_variable_name} = '
                f'cpy({self.variable_name});'
            )
        ap.append_js_expression(expression=expression)



"""Implementations for Sprite class.
"""

from typing import Optional

from apysc._display import graphics
from apysc._display.child_interface import ChildInterface
from apysc._display.display_object import DisplayObject
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.revert_interface import RevertInterface


class Sprite(DisplayObject, ChildInterface, RevertInterface):
    """
    This class is for the basic display object that
    can be a parent.

    References
    ----------
    - Sprite document
        - https://simon-ritchie.github.io/apysc/sprite.html

    Examples
    --------
    >>> import apysc as ap
    >>> stage: ap.Stage = ap.Stage()
    >>> sprite_1: ap.Sprite = ap.Sprite()
    >>> # Create the sprite child rectangle
    >>> sprite_1.graphics.begin_fill(color='#0af')
    >>> rect: ap.Rectangle = sprite_1.graphics.draw_rect(
    ...     x=50, y=50, width=50, height=50)
    >>> sprite_1.graphics.contains(rect)
    Boolean(True)

    >>> # Move the created rectangle to the other sprite
    >>> sprite_2: ap.Sprite = ap.Sprite()
    >>> sprite_2.add_child(rect)
    >>> sprite_1.graphics.contains(rect)
    Boolean(False)
    >>> sprite_2.contains(rect)
    Boolean(True)

    >>> # Move the sprite container
    >>> sprite_2.x = ap.Int(50)
    >>> sprite_2.x
    Int(50)
    """

    graphics: 'graphics.Graphics'

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='Sprite')
    def __init__(
            self, *,
            variable_name: Optional[str] = None) -> None:
        """
        Create a basic display object that can be a parent.

        Parameters
        ----------
        variable_name : str or None, default None
            Variable name of this instance. A js expression uses
            this setting. It is unnecessary to specify any
            string except when instantiating the `Sprite` subclass.

        References
        ----------
        - Sprite document
            - https://simon-ritchie.github.io/apysc/sprite.html

        Examples
        --------
        >>> import apysc as ap
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite_1: ap.Sprite = ap.Sprite()
        >>> # Create the sprite child rectangle
        >>> sprite_1.graphics.begin_fill(color='#0af')
        >>> rect: ap.Rectangle = sprite_1.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> sprite_1.graphics.contains(rect)
        Boolean(True)

        >>> # Move the created rectangle to the other sprite
        >>> sprite_2: ap.Sprite = ap.Sprite()
        >>> sprite_2.add_child(rect)
        >>> sprite_1.graphics.contains(rect)
        Boolean(False)
        >>> sprite_2.contains(rect)
        Boolean(True)

        >>> # Move the sprite container
        >>> sprite_2.x = ap.Int(50)
        >>> sprite_2.x
        Int(50)
        """
        import apysc as ap
        from apysc._expression import expression_variables_util
        from apysc._expression import var_names
        stage: ap.Stage = ap.get_stage()
        if variable_name is None:
            variable_name = expression_variables_util.\
                get_next_variable_name(type_name=var_names.SPRITE)
        self._children = ap.Array([])
        super(Sprite, self).__init__(variable_name=variable_name)
        self._append_constructor_expression()
        self.graphics = graphics.Graphics(parent=self)
        stage.add_child(child=self)
        self._set_overflow_visible_setting()

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='Sprite')
    def _append_constructor_expression(self) -> bool:
        """
        Append Sprite constructor expression.

        Notes
        -----
        This interface does not add an expression if this instance is the
        `Sprite`' subclass.

        Returns
        -------
        appended : bool
            If this interface appends an expression, this interface
            returns True.
        """
        import apysc as ap
        from apysc._display.stage import get_stage_variable_name
        from apysc._type import type_util
        is_same_class_instance: bool = type_util.is_same_class_instance(
            class_=Sprite, instance=self)
        if not is_same_class_instance:
            return False
        stage_variable_name: str = get_stage_variable_name()
        expression: str = (
            f'\nvar {self.variable_name} = {stage_variable_name}.nested();'
        )
        ap.append_js_expression(expression=expression)
        return True

    def _make_snapshot(self, *, snapshot_name: str) -> None:
        """
        Make values' snapshot.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self.graphics._run_all_make_snapshot_methods(
            snapshot_name=snapshot_name)

    def _revert(self, *, snapshot_name: str) -> None:
        """
        Revert values if snapshot exists.

        Parameters
        ----------
        snapshot_name : str
            Target snapshot name.
        """
        if not self._snapshot_exists(snapshot_name=snapshot_name):
            return
        self.graphics._run_all_revert_methods(snapshot_name=snapshot_name)

    def __repr__(self) -> str:
        """
        Get a string representation of this instance (for the sake of
        debugging).

        Returns
        -------
        repr_str : str
            Type name and variable name will be set
            (e.g., `Sprite('<variable_name>')`).
        """
        repr_str: str = f"Sprite('{self.variable_name}')"
        return repr_str



"""This module is for the translation mapping data of the
following document:

Document file: graphics_fill_alpha.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Graphics fill_alpha interface':
    '',

    'This page explains the `Graphics` class `fill_alpha` property interface.':  # noqa
    '',

    '## What interface is this?':
    '',

    'The `fill_alpha` property interface updates or get the instance\'s fill alpha (opacity).':  # noqa
    '',

    '## Basic usage':
    '',

    'The getter or setter interface value becomes (or require) the `Number` value (0.0 to 1.0).\n\nThe following example sets the 0.5 fill alpha to the second rectangle and 0.25 to the third rectangle:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=350, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nsprite.graphics.begin_fill(color=\'#0af\')\n\nrectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n    x=50, y=50, width=50, height=50)\n\nrectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n    x=150, y=50, width=50, height=50)\nrectangle_2.fill_alpha = ap.Number(0.5)\n\nrectangle_3: ap.Rectangle = sprite.graphics.draw_rect(\n    x=250, y=50, width=50, height=50)\nrectangle_3.fill_alpha = ap.Number(0.25)\n\nap.save_overall_html(\n    dest_dir_path=\'./graphics_fill_alpha_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_fill_alpha_basic_usage/index.html" width="350" height="150"></iframe>':  # noqa
    '',

    '## fill_alpha property API':
    '',

    '<!-- Docstring: apysc._display.fill_alpha_interface.FillAlphaInterface.fill_alpha -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get this instance\'s fill opacity.<hr>\n\n**[Returns]**\n\n- `fill_alpha`: Number\n  - Current fill opacity (0.0 to 1.0).\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> rectangle.fill_alpha = ap.Number(0.5)\n>>> rectangle.fill_alpha\nNumber(0.5)\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: stage.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Stage':
    '',

    'This page explains the `Stage` class.':
    '',

    '## What is the Stage?':
    '',

    'The `Stage` is the apysc overall drawing area (like a viewport, canvas, or something else) and contains all elements.\n\nYou must create the `Stage` at the first of the apysc project (this runs cleaning up internal data and files).':  # noqa
    '',

    '## Create stage':
    '',

    'Creating stage is simple, like this:':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage()\n```':  # noqa
    '',

    '':
    '',

    '## Stage background color setting':
    '',

    '`Stage` class has a `background_color` option, which changes the stage\'s background color.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=\'#333\',\n    stage_elem_id=\'stage\')\n\nap.save_overall_html(\n    dest_dir_path=\'stage_background_color/\')\n```':  # noqa
    '',

    'This will create HTML with black background stage, as follows:\n\n<iframe src="static/stage_background_color/index.html" width="300" height="185"></iframe>':  # noqa
    '',

    '## Stage size setting':
    '',

    'Stage class has options to set stage width and stage height (arguments of `stage_width` and `stage_height`). These settings change stage sizes.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    stage_width=500, stage_height=50,\n    background_color=\'#333\',\n    stage_elem_id=\'stage\')\n\nap.save_overall_html(\n    dest_dir_path=\'stage_size/\')\n```':  # noqa
    '',

    'The Previous script will create a horizontal stage, as follows:\n\n<iframe src="static/stage_size/index.html" width="500", height="50"></iframe>':  # noqa
    '',

    '## Stage element id setting':
    '',

    'Stage element id (HTML id) can be set by `stage_elem_id` argument. If you don\'t specify this, the apysc sets any unique id (based on created timestamp, like `stage_12345...`).':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstage: ap.Stage = ap.Stage(\n    background_color=\'#333\',\n    stage_elem_id=\'line_chart_1\')\n```':  # noqa
    '',

    'This option is useful when using the apysc project multiple times (for an easily identifiable ID) or version control.':  # noqa
    '',

    '## Stage class constructor API':
    '',

    '<!-- Docstring: apysc._display.stage.Stage.__init__ -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `__init__(self, *, stage_width:int=300, stage_height:int=185, background_color:str=\'#ffffff\', add_to:str=\'body\', stage_elem_id:Union[str, NoneType]=None) -> None`<hr>\n\n**[Interface summary]** Create Stage (overall viewport) instance.<hr>\n\n**[Parameters]**\n\n- `stage_width`: int, default 300\n  - Stage width.\n- `stage_height`: int, default 185\n  - Stage height\n- `background_color`: str, default \'#ffffff\'\n  - Hexadecimal background color string.\n- `add_to`: str, default \'body\'\n  - Specification of element to add stage. Unique tag (e.g., \'body\') or ID selector (e.g., \'#any-unique-elem\') is acceptable.\n- `stage_elem_id`: str or None, optional\n  - ID attribute set to stage html element (e.g., \'line-graph\'). If None is set, random integer will be applied.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     stage_width=500, stage_height=300,\n...     background_color=\'#333\', stage_elem_id=\'sales_chart\')\n```':  # noqa
    '',

    '':
    '',

    '## stage_elem_id property API':
    '',

    '<!-- Docstring: apysc._display.stage.Stage.stage_elem_id -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get stage\'s html element id.<hr>\n\n**[Returns]**\n\n- `stage_elem_id`: str\n  - Stage\'s html element id (not including class or id symbol). e.g., \'line-graph\'\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage(\n...     stage_width=500, stage_height=300,\n...     background_color=\'#333\', stage_elem_id=\'sales_chart\')\n>>> stage.stage_elem_id\n\'sales_chart\'\n```':  # noqa
    '',

    '':
    '',

    '## add_child API':
    '',

    '<!-- Docstring: apysc._display.child_interface.ChildInterface.add_child -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `add_child(self, child:apysc._display.display_object.DisplayObject) -> None`<hr>\n\n**[Interface summary]** Add display object child to this instance.<hr>\n\n**[Parameters]**\n\n- `child`: DisplayObject\n  - Child instance to add.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite_1: ap.Sprite = ap.Sprite()\n>>> sprite_1.graphics.begin_fill(color=\'#0af\')\n>>> rectangle: ap.Rectangle = sprite_1.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> sprite_2: ap.Sprite = ap.Sprite()\n>>> sprite_2.add_child(rectangle)\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [add_child and remove_child interfaces document](https://simon-ritchie.github.io/apysc/add_child_and_remove_child.html)':  # noqa
    '',

    '## remove_child API':
    '',

    '<!-- Docstring: apysc._display.child_interface.ChildInterface.remove_child -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `remove_child(self, child:apysc._display.display_object.DisplayObject) -> None`<hr>\n\n**[Interface summary]** Remove display object child from this instance.<hr>\n\n**[Parameters]**\n\n- `child`: DisplayObject\n  - Child instance to remove.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\', alpha=0.5)\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> sprite.graphics.remove_child(rectangle)\n>>> print(rectangle.parent)\nNone\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [add_child and remove_child interfaces document](https://simon-ritchie.github.io/apysc/add_child_and_remove_child.html)':  # noqa
    '',

    '## contains API':
    '',

    '<!-- Docstring: apysc._display.child_interface.ChildInterface.contains -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `contains(self, child:apysc._display.display_object.DisplayObject) -> apysc._type.boolean.Boolean`<hr>\n\n**[Interface summary]** Get a boolean whether this instance contains a specified child.<hr>\n\n**[Parameters]**\n\n- `child`: DisplayObject\n  - Child instance to check.\n\n<hr>\n\n**[Returns]**\n\n- `result`: Boolean\n  - If this instance contains a specified child, this method returns True.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\', alpha=0.5)\n>>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> sprite.graphics.contains(rectangle)\nBoolean(True)\n\n>>> rectangle.remove_from_parent()\n>>> sprite.graphics.contains(rectangle)\nBoolean(False)\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [contains interface document](https://simon-ritchie.github.io/apysc/contains.html)':  # noqa
    '',

    '## num_children property API':
    '',

    '<!-- Docstring: apysc._display.child_interface.ChildInterface.num_children -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get a current children\'s number.<hr>\n\n**[Returns]**\n\n- `num_children`: int\n  - Current children number.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\', alpha=0.5)\n>>> rectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> rectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=150, y=50, width=50, height=50)\n>>> sprite.graphics.num_children\nInt(2)\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [num_children interface document](https://simon-ritchie.github.io/apysc/num_children.html)':  # noqa
    '',

    '## get_child_at API':
    '',

    '<!-- Docstring: apysc._display.child_interface.ChildInterface.get_child_at -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `get_child_at(self, index:Union[int, apysc._type.int.Int]) -> apysc._display.display_object.DisplayObject`<hr>\n\n**[Interface summary]** Get child at a specified index.<hr>\n\n**[Parameters]**\n\n- `index`: int or Int\n  - Child\'s index (start from 0).\n\n<hr>\n\n**[Returns]**\n\n- `child`: DisplayObject\n  - Target index child instance.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.begin_fill(color=\'#0af\', alpha=0.5)\n>>> rectangle_1: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=50, y=50, width=50, height=50)\n>>> rectangle_2: ap.Rectangle = sprite.graphics.draw_rect(\n...     x=150, y=50, width=50, height=50)\n>>> child_at_index_1: ap.DisplayObject = (\n...     sprite.graphics.get_child_at(1))\n>>> child_at_index_1 == rectangle_2\nTrue\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [get_child_at interface document](https://simon-ritchie.github.io/apysc/get_child_at.html)':  # noqa
    '',

}



"""Handler circular calling related utilities.
"""

from typing import List
from typing import Optional
from typing import Tuple


def is_handler_circular_calling(*, handler_name: str) -> bool:
    """
    Get a boolean value whether a specified handler is
    a circular call or not.

    Parameters
    ----------
    handler_name : str
        Targer handler name.

    Returns
    -------
    result : bool
        If a specified handler is a circular call, this interface
        returns True.
    """
    from apysc._expression import event_handler_scope
    if _is_already_saved_circular_calling(handler_name=handler_name):
        return True
    original_handler_name: str = handler_name
    handler_name = event_handler_scope.remove_suffix_num_from_handler_name(
        handler_name=handler_name)
    handler_names: List[str] = _read_handler_names()
    handler_names = _append_handler_name_to_last_of_list(
        handler_name=handler_name, handler_names=handler_names)
    count: int = handler_names.count(handler_name)
    if count < 2:
        return False
    prev_handler_name: str = handler_names[-2]
    prev_handler_count: int = 0
    for i, handler_name_ in enumerate(handler_names):
        if i == 0:
            continue
        if handler_name_ != handler_name:
            continue
        prev_handler_name_: str = handler_names[i - 1]
        if prev_handler_name_ != prev_handler_name:
            continue
        prev_handler_count += 1
        if prev_handler_count == 2:
            break
    if prev_handler_count == 2:
        _save_circular_calling_handler_name(
            handler_name=original_handler_name)
        return True
    return False


def _is_already_saved_circular_calling(*, handler_name: str) -> bool:
    """
    Get a boolean indicating whether the interface
    already has saved a handler name as the circular calling handler or not.

    Parameters
    ----------
    handler_name : str
        Target handler's name.

    Returns
    -------
    result : bool
        If the interface already saves a specified handler
        name as the circular calling handler, this interface
        returns True.
    """
    prev_handler_name: str = get_prev_handler_name(
        handler_name=handler_name)
    if prev_handler_name == '':
        return False
    return True


def get_prev_handler_name(*, handler_name: str) -> str:
    """
    Get a previous handler's name of a specified handler's
    one if it is a circular calling handler.

    Parameters
    ----------
    handler_name : str
        Target handler's name.

    Returns
    -------
    prev_handler_name : str
        A previous handler's name. If there is no
        previous one, this interface returns a blank string.
    """
    from apysc._expression import expression_data_util
    table_name: str = expression_data_util.TableName.\
        CIRCULAR_CALLING_HANDLER_NAME.value
    query: str = (
        f'SELECT prev_handler_name FROM {table_name} '
        f"WHERE handler_name = '{handler_name}';"
    )
    expression_data_util.exec_query(sql=query)
    result: Optional[Tuple[str]] = expression_data_util.cursor.fetchone()
    if result is None:
        return ''
    return result[0]


def get_prev_variable_name(*, handler_name: str) -> str:
    """
    Get a previous handler binding instance's variable
    name if a specified handler is a circular calling's handler.

    Parameters
    ----------
    handler_name : str
        Target handler's name.

    Returns
    -------
    prev_variable_name : str
        A previous handler binding instance's variable name.
        If there is no previous (same handler's name prefix) one,
        this interface returns a blank string.
    """
    from apysc._expression import expression_data_util
    table_name: str = expression_data_util.TableName.\
        CIRCULAR_CALLING_HANDLER_NAME.value
    query: str = (
        f'SELECT prev_variable_name FROM {table_name} '
        f"WHERE handler_name = '{handler_name}';"
    )
    expression_data_util.exec_query(sql=query)
    result: Optional[Tuple[str]] = expression_data_util.cursor.fetchone()
    if result is None:
        return ''
    return result[0]


def _save_circular_calling_handler_name(*, handler_name: str) -> None:
    """
    Save a circular calling handler name to the SQLite.

    Parameters
    ----------
    handler_name : str
        Target handler's name.
    """
    from apysc._expression import expression_data_util
    prev_hadler_name: str = _get_same_name_prev_hadler_name(
        handler_name=handler_name)
    prev_variable_name: str = _get_same_name_prev_variable_name(
        handler_name=handler_name)
    table_name: str = expression_data_util.TableName.\
        CIRCULAR_CALLING_HANDLER_NAME.value
    query: str = (
        f'INSERT INTO {table_name}'
        '(handler_name, prev_handler_name, prev_variable_name) '
        f"VALUES('{handler_name}', '{prev_hadler_name}', "
        f"'{prev_variable_name}');"
    )
    expression_data_util.exec_query(sql=query)


def _get_same_name_prev_hadler_name(*, handler_name: str) -> str:
    """
    Get the same previous name (but the suffix number is
    different) handler's name from the current stack.

    Parameters
    ----------
    handler_name : str
        Target handler's name.

    Returns
    -------
    same_name_prev_hadler_name : str
        A previous same name (but the suffix number is different)
        handler's name.
    """
    same_name_prev_hadler_name: str
    same_name_prev_hadler_name, _ = _get_same_name_prev_data(
        handler_name=handler_name)
    return same_name_prev_hadler_name


def _get_same_name_prev_variable_name(*, handler_name: str) -> str:
    """
    Get the same previous name (but the suffix number is
    different) handler's binding variable name from the
    current stack.

    Parameters
    ----------
    handler_name : str
        Target handler's name.

    Returns
    -------
    prev_variable_name : str
        A previous handler that binding instance's
        variable name.
    """
    prev_variable_name: str
    _, prev_variable_name = _get_same_name_prev_data(
        handler_name=handler_name)
    return prev_variable_name


def _get_same_name_prev_data(*, handler_name: str) -> Tuple[str, str]:
    """
    Get a previous handler name and variable name values
    of the same previous name (but the suffix number
    is different) handler from the current stack.

    Parameters
    ----------
    handler_name : str
        Target handler's name.

    Returns
    -------
    prev_hadler_name : str
        A previous same name (but the suffix number is different)
        handler's name value.
    prev_variable_name : str
        A previous variable name value.

    Raises
    ------
    ValueError
        If there is no previous same name handler's name in the SQLite.
    """
    from apysc._expression import event_handler_scope
    from apysc._expression import expression_data_util
    table_name: str = expression_data_util.TableName.\
        HANDLER_CALLING_STACK.value
    query: str = (
        f'SELECT handler_name, variable_name FROM {table_name} '
        f'ORDER BY scope_count DESC'
    )
    expression_data_util.exec_query(sql=query)
    result: List[Tuple[str, str]] = expression_data_util.cursor.fetchall()
    for i, tpl in enumerate(result):
        handler_name_: str = tpl[0]
        if i == 0 and handler_name == handler_name_:
            continue
        no_suffix_handler_name: str = event_handler_scope.\
            remove_suffix_num_from_handler_name(handler_name=handler_name)
        no_suffix_handler_name_: str = event_handler_scope.\
            remove_suffix_num_from_handler_name(handler_name=handler_name_)
        if no_suffix_handler_name != no_suffix_handler_name_:
            continue
        prev_hadler_name: str = handler_name_
        prev_variable_name: str = tpl[1]
        return prev_hadler_name, prev_variable_name
    raise ValueError(
        'Previous same name handler does not exitst in the SQLite.'
        ' Please check the implementation of this function\'s calling.')


def _append_handler_name_to_last_of_list(
        *, handler_name: str, handler_names: List[str]) -> List[str]:
    """
    Append a specified handler's name to the list
    last if the last one is the other handler's name.
    This function is for the unifying last value regardless
    of the `HandlerScope` setting.

    Parameters
    ----------
    handler_name : str
        Target handler name.
    handler_names : list of str
        List to be appended.

    Returns
    -------
    handler_names : list of str
        Result list value.
    """
    if not handler_names:
        return [handler_name]
    if handler_names[-1] == handler_name:
        return handler_names
    handler_names.append(handler_name)
    return handler_names


def _read_handler_names() -> List[str]:
    """
    Read the current handler names from the calling stack.

    Returns
    -------
    handler_names : list of str
        Target handler names.
    """
    from apysc._expression import event_handler_scope
    from apysc._expression import expression_data_util
    table_name: str = expression_data_util.TableName.\
        HANDLER_CALLING_STACK.value
    query: str = (
        f'SELECT handler_name FROM {table_name} '
        f'ORDER BY scope_count'
    )
    expression_data_util.exec_query(sql=query)
    result: List[Tuple[str]] = expression_data_util.cursor.fetchall()
    handler_names: List[str] = [
        event_handler_scope.remove_suffix_num_from_handler_name(
            handler_name=tpl[0])
        for tpl in result]
    return handler_names



"""Class implementation for the stop_propagation interface.
"""

from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.variable_name_interface import VariableNameInterface


class StopPropagationInterface(VariableNameInterface):

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='StopPropagationInterface')
    def stop_propagation(self) -> None:
        """
        Stop event propagation.

        References
        ----------
        - Event class prevent_default and stop_propagation interfaces document
            - https://simon-ritchie.github.io/apysc/event_prevent_default_and_stop_propagation.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> def on_click(e: ap.MouseEvent, options: dict) -> None:
        ...     e.stop_propagation()
        ...     ap.trace('Clicked!')
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> _ = sprite.click(on_click)
        >>> _ = rectangle.click(on_click)
        """
        import apysc as ap
        expression: str = (
            f'{self.variable_name}.stopPropagation();'
        )
        ap.append_js_expression(expression=expression)



"""This module is for translation-mapping
utility implementations.
"""

import importlib
import os
import re
from types import ModuleType
from typing import Dict
from typing import List
from typing import Match
from typing import Optional
from typing import Pattern
from typing import Union

from apysc._lint_and_doc.docs_lang import Lang
from apysc._lint_and_doc.docstring_util import DOCSTRING_PATH_COMMENT_PATTERN
from apysc._lint_and_doc.document_text_split_util import BodyText
from apysc._lint_and_doc.document_text_split_util import CodeBlock
from apysc._lint_and_doc.document_text_split_util import Heading

MAPPING_CONST_NAME: str = 'MAPPING'

SKIPPING_PATTERNS: List[Pattern] = [
    re.compile(pattern=DOCSTRING_PATH_COMMENT_PATTERN),
]

_HR_TAG_PATTERN: str = r'^<hr>$'

_INTERFACE_SIGNATURE_PATTERN: str = (
    r'\*\*\[Interface signature\]\*\* .*?<hr>'
)

_CODE_BLOCK_IFRAME_PATTERN: str = (
    r'<iframe src="static.*?index\.html".*?></iframe>'
)

MAPPING_UNNECESSARY_PATTERNS: List[Pattern] = [
    re.compile(pattern=_HR_TAG_PATTERN,),
    re.compile(pattern=_INTERFACE_SIGNATURE_PATTERN),
    re.compile(pattern=_CODE_BLOCK_IFRAME_PATTERN),
]

_SplittedVals = List[Union[Heading, BodyText, CodeBlock]]


def read_mapping_data(
        *, src_doc_file_path: str,
        lang: Lang) -> Dict[str, str]:
    """
    Read an already saved mapping data.

    Parameters
    ----------
    src_doc_file_path : str
        A target source document file path.
    lang : Lang
        A target translation language.

    Returns
    -------
    already_saved_mapping : dict
        An already saved mapping data dictionary.
    """
    from apysc._file import module_util
    mapping_module_path: str = get_mapping_module_path(
        src_doc_file_path=src_doc_file_path, lang=lang)
    if not os.path.isfile(mapping_module_path):
        return {}
    module: ModuleType = module_util.read_target_path_module(
        module_path=mapping_module_path)
    importlib.reload(module)
    if not hasattr(module, MAPPING_CONST_NAME):
        return {}
    already_saved_mapping: Dict[str, str] = getattr(
        module, MAPPING_CONST_NAME)
    return already_saved_mapping


def get_mapping_module_path(
        *, src_doc_file_path: str, lang: Lang) -> str:
    """
    Get a mapping data module path.

    Parameters
    ----------
    src_doc_file_path : str
        A target source document file path.
    lang : Lang
        A target translation language.

    Returns
    -------
    mapping_module_path : str
        A mapping data module path.
    """
    basename: str = os.path.basename(src_doc_file_path)
    basename = basename.replace('.md', '.py', 1)
    mapping_module_path: str = os.path.join(
        f'./apysc/_translation/{lang.value}/',
        basename,
    )
    return mapping_module_path


def convert_splitted_values_to_keys(
        *,
        splitted_values: _SplittedVals) -> List[str]:
    """
    Convert specified splitted values to dictionary's key
    strings.

    Parameters
    ----------
    splitted_values : List of Heading, BodyText and CodeBlock
        Target splitted values.

    Returns
    -------
    keys : list of str
        Converted strings.
    """
    keys: List[str] = []
    for splitted_value in splitted_values:
        is_body_text: bool = False
        key: str = ''
        if isinstance(splitted_value, Heading):
            key = splitted_value.overall_text
        if isinstance(splitted_value, BodyText):
            is_body_text = True
            key = splitted_value.text
        if isinstance(splitted_value, CodeBlock):
            key = splitted_value.overall_code_block
        key = escape_key_or_value(key_or_val=key)
        if not is_body_text:
            keys.append(key)
        else:
            _append_body_text_keys_to_list(key=key, keys=keys)
    return keys


def _append_body_text_keys_to_list(
        *, key: str, keys: List[str]) -> None:
    """
    Append document's body text keys to a specified key's list.

    Parameters
    ----------
    key : str
        A target key string. This interface splits its string
        if there are two consecutive line breaks.
    keys : list of str
        A key's list to append.
    """
    splitted_keys: List[str] = key.split('\\n\\n')
    for key_ in splitted_keys:
        keys.append(key_)


def escape_key_or_value(*, key_or_val: str) -> str:
    """
    Escape a mapping key or value to save.

    Parameters
    ----------
    key_or_val : str
        A target key or value.

    Returns
    -------
    key_or_val : str
        An escaped key or value string.
    """
    key_or_val = key_or_val.replace('\\', '\\\\')
    key_or_val = key_or_val.replace("'", "\\'")
    key_or_val = key_or_val.replace('\n', '\\n')
    key_or_val = key_or_val.replace('\\\\n', '\\n')
    return key_or_val


def remove_escaping_from_key_or_value(*, key_or_val: str) -> str:
    """
    Remove escaping characters from a specified key or value string.

    Parameters
    ----------
    key_or_val : str
        A target key or value string.

    Returns
    -------
    key_or_val : str
        A result key or value string.
    """
    key_or_val = key_or_val.replace('\\\\', '\\')
    key_or_val = key_or_val.replace("\\'", "'")
    key_or_val = key_or_val.replace('\\n', '\n')
    return key_or_val


def is_mapping_unnecessary_key(*, key: str) -> bool:
    """
    Get a boolean indicating whether a specified key
    is an unnecessary mapping pattern or not.

    Parameters
    ----------
    key : str
        A target key string to check.

    Returns
    -------
    result : bool
        This interface returns True if a specified key is
        an unnecessary mapping pattern.
    """
    for pattern in MAPPING_UNNECESSARY_PATTERNS:
        match: Optional[Match] = pattern.search(string=key)
        if match is not None:
            return True
    return False


def is_translation_skipping_key(*, key: str) -> bool:
    """
    Get a boolean indicating whether a specified
    key is a translation skipping pattern or not.

    Parameters
    ----------
    key : str
        A target key string to check.

    Returns
    -------
    result : bool
        This interface returns True if a specified key
        is a translation skipping pattern.
    """
    for pattern in SKIPPING_PATTERNS:
        match: Optional[Match] = pattern.search(string=key)
        if match is not None:
            return True
    return False


def get_translated_file_path_from_src_path(
        *, source_doc_path: str,
        lang: Lang) -> str:
    """
    Get a translated file path from a specified source
    document's file path.

    Parameters
    ----------
    source_doc_path : str
        A target document's file path.
    lang : Lang
        A target language.

    Returns
    -------
    translated_file_path : str
        A translated file path.
    """
    basename: str = os.path.basename(source_doc_path)
    basename = f'{lang.value}_{basename}'
    dir_path = os.path.dirname(source_doc_path)
    translated_file_path: str = os.path.join(dir_path, basename)
    return translated_file_path



"""This module is for the data model of the fixed-translation
mapping settings.
"""

import inspect
import os
from functools import lru_cache
from types import ModuleType
from typing import List
from typing import Optional
from typing import Tuple

from apysc._lint_and_doc.docs_lang import Lang


class Mapping:
    """This class is for a single fixed-translation mapping setting.
    """

    _key: str
    _value: str

    def __init__(self, *, key: str, value: str) -> None:
        """
        A single fixed-translation mapping setting.

        Parameters
        ----------
        key : str
            A key value (this value needs to set source
            English string).
        value : str
            A translated value.
        """
        self._key = key
        self._value = value

    @property
    def key(self) -> str:
        """
        Get a key value (a source english string).

        Returns
        -------
        key : str
            A key value (a source english string).
        """
        return self._key

    @property
    def value(self) -> str:
        """
        Get a translated value.

        Returns
        -------
        value : str
            A translated value.
        """
        return self._value


class Mappings:
    """This class is for fixed-translation mappings settings.
    """

    mappings: List[Mapping]

    def __init__(self, mappings: List[Mapping]) -> None:
        """
        The class for fixed-translation mappings settings.

        Parameters
        ----------
        mappings : list of Mapping
            A target mappings list.
        """
        self.mappings = mappings


def get_fixed_translation_str_if_exists(
        *, key: str, lang: Lang) -> str:
    """
    If a mapping setting exists, get a fixed-translation
    string from a specified language key string.

    Parameters
    ----------
    key : str
        A target key string (source English string).
    lang : Lang
        A target language.

    Returns
    -------
    translation_str : str
        A translated string. If there is no fixed-translation
        mapping setting, this interface returns a blank string.
    """
    mappings: Optional[Mappings] = _read_mappings(lang=lang)
    if mappings is None:
        return ''
    for mapping in mappings.mappings:
        if mapping.key == key:
            return mapping.value
    return ''


@lru_cache(maxsize=None)
def _read_mappings(*, lang: Lang) -> Optional[Mappings]:
    """
    Read a fixed-translation mappings settings if it exists.

    Parameters
    ----------
    lang : Lang
        A target language.

    Returns
    -------
    mappings : Mappings or None
        A read mappings settings. If there is no mappings
        settings, this interface returns None.
    """
    from apysc._file import module_util
    module_path: str = _get_mappings_module_path_from_lang(lang=lang)
    if not os.path.isfile(module_path):
        return None
    module: ModuleType = module_util.read_target_path_module(
        module_path=module_path)
    members: List[Tuple[str, Mappings]] = inspect.getmembers(
        module, predicate=lambda x: isinstance(x, Mappings))
    if not members:
        return None
    return members[0][1]


def _get_mappings_module_path_from_lang(*, lang: Lang) -> str:
    """
    Get a fixed-translation mappings settings module path
    of a specified language.

    Parameters
    ----------
    lang : Lang
        A target language.

    Returns
    -------
    module_path : str
        A fixed-translation mappings settings module path
        of a specified language.
    """
    module_path: str = (
        './apysc/_lint_and_doc/fixed_translation_mapping/'
        f'{lang.value}.py'
    )
    return module_path



"""This module is for the translation mapping data of the
following document:

Document file: unset_debug_mode.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# unset_debug_mode interface':
    '',

    'This page explains the `unset_debug_mode` function interface.':
    '',

    '## What interface is this?':
    '',

    'The `unset_debug_mode` function interface unsets the debug mode setting. It stops the debug information appending.\n\nThe debug mode exports lots of information. Sometimes it becomes cumbersome. In that case, stopping the debug mode is helpful when it is no longer needed.':  # noqa
    '',

    '## Notes':
    '',

    'If the exporting interface (e.g., `ap.save_overall_html`) `minify` option is enabled, it removes debug mode information. So it is required to set the `minify=False` when you use the `unset_debug_mode` interface.':  # noqa
    '',

    '## Basic usage':
    '',

    'The `unset_debug_mode` interface requires no arguments.\n\nThe following example appends the debug information only at the `int_1` instantiation and incremental addition of 10.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    stage_width=150, stage_height=150, background_color=\'#333\',\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\nap.set_debug_mode()\nint_1: ap.Int = ap.Int(10)\nint_1 += 10\nap.unset_debug_mode()\nint_2: ap.Int = ap.Int(20)\nint_2 += 20\n\nap.save_overall_html(\n    minify=False, dest_dir_path=\'unset_debug_mode_basic_usage/\')\n```':  # noqa
    '',

    'The exported HTML includes the debug information at the first integer position. It doesn\'t include the sprite and second integer positions, as follows:':  # noqa
    '',

    '```js\n...\n  var sp_1 = stage.nested();\n  var g_1 = stage.nested();\n  arr_2.push(g_1);\n  var i_12 = -1;\n  i_12 = arr_2.indexOf(g_1);\n  var b_3 = false;\n  var i_13 = -1;\n...\n  //////////////////////////////////////////////////////////////////////\n  // [__init__ 12] started.\n  // module name: apysc._type.int\n  // class: Int\n  // arguments and variables:\n  //    value = 10\n  //    self = 0()\n    //////////////////////////////////////////////////////////////////////\n    // [__init__ 14] started.\n    // module name: apysc._type.number_value_interface\n    // class: NumberValueInterface\n    // arguments and variables:\n    //    type_name = \'i\'\n    //    value = 10\n    //    self = 0(i_16)\n    // [__init__ 14] ended.\n    // module name: apysc._type.number_value_interface\n    // class: NumberValueInterface\n    //////////////////////////////////////////////////////////////////////\n    //////////////////////////////////////////////////////////////////////\n    // [to_int_from_float 14] started.\n    // module name: apysc._converter.cast\n    // arguments and variables:\n    //    int_or_float = 10\n    // [to_int_from_float 14] ended.\n    // module name: apysc._converter.cast\n    //////////////////////////////////////////////////////////////////////\n    //////////////////////////////////////////////////////////////////////\n    // [append_constructor_expression 14] started.\n    // module name: apysc._type.number_value_interface\n    // class: NumberValueInterface\n    // arguments and variables:\n    //    self = 10(i_16)\n      var i_16 = 10;\n    // [append_constructor_expression 14] ended.\n    // module name: apysc._type.number_value_interface\n    // class: NumberValueInterface\n    //////////////////////////////////////////////////////////////////////\n...\n  var i_18 = 20;\n  var i_19 = cpy(i_18);\n  var i_19 = i_18 + 20;\n  i_18 = i_19;\n...\n```':  # noqa
    '',

    '':
    '',

    '## See also':
    '',

    '- [Set debug mode interface](set_debug_mode.md)':
    '',

    '## unset_debug_mode API':
    '',

    '<!-- Docstring: apysc._html.debug_mode.unset_debug_mode -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `unset_debug_mode() -> None`<hr>\n\n**[Interface summary]** Unset the debug mode for the HTML and JavaScript debugging.<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> ap.set_debug_mode()\n>>> int_val: ap.Int = ap.Int(10)\n>>> ap.unset_debug_mode()\n```':  # noqa
    '',

}



"""This module is for the translation mapping data of the
following document:

Document file: array.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Array':
    '',

    'This page explains the `Array` class.\n\nBefore reading on, maybe it is helpful to read the following page:\n\n- [Why the apysc library doesn\'t use the Python built-in data type](why_apysc_doesnt_use_python_builtin_data_type.md)':  # noqa
    '',

    '## What is the Array?':
    '',

    'The `Array` class is the apysc array class. It behaves like the Python built-in `list` value.':  # noqa
    '',

    '## Constructor method':
    '',

    'The `Array` class constructor method requires iterable objects, like the `list`\\, `tuple`\\, `range`\\, or `Array` value.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\narr_from_list: ap.Array = ap.Array([1, 2, 3])\nassert arr_from_list == [1, 2, 3]\n\narr_from_tuple: ap.Array = ap.Array((4, 5, 6))\nassert arr_from_tuple == [4, 5, 6]\n\nother_arr: ap.Array = ap.Array([7, 8, 9])\narr_from_arr: ap.Array = ap.Array(other_arr)\nassert arr_from_arr == [7, 8, 9]\n```':  # noqa
    '',

    '':
    '',

    '## Generic type':
    '',

    'If the `Array` values types are unique, you can set the generic type to an `Array` value. This annotation may be helpful when you use it on the IDE (for type checkers).':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\narr: ap.Array[int] = ap.Array([1, 2])\nint_val: int = arr.pop()\nassert isinstance(int_val, int)\n```':  # noqa
    '',

    '':
    '',

    '## See also':
    '',

    '- [Funcdamental data classes common value interface](fundamental_data_classes_value_interface.md)\n- [Array class append and push interfaces](array_append_and_push.md)\n- [Array class extend and concat interfaces](array_extend_and_concat.md)\n- [Array class insert and insert at interfaces](array_insert_and_insert_at.md)\n- [Array class pop interface](array_pop.md)\n- [Array class remove and remove at interfaces](array_remove_and_remove_at.md)\n- [Array class sort interface](array_sort.md)\n- [Array class reverse interface](array_reverse.md)\n- [Array class slice interface](array_slice.md)\n- [Array class length interface](array_length.md)\n- [Array class join interface](array_join.md)\n- [Array class index of interface](array_index_of.md)\n- [Array class comparison interfaces](array_comparison.md)':  # noqa
    '',

    '## Array class constructor API':
    '',

    '<!-- Docstring: apysc._type.array.Array.__init__ -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `__init__(self, value:Union[List[~T], tuple, range, _ForwardRef(\'Array\')]) -> None`<hr>\n\n**[Interface summary]** Array class for the apysc library.<hr>\n\n**[Parameters]**\n\n- `value`: Array or list or tuple or range\n  - Initial array value.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr\nArray([1, 2, 3])\n\n>>> arr[0]\n1\n\n>>> arr[1]\n2\n\n>>> arr = ap.Array((4, 5, 6))\n>>> arr\nArray([4, 5, 6])\n\n>>> arr = ap.Array(range(3))\n>>> arr\nArray([0, 1, 2])\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [Array class comparison interfaces document](https://simon-ritchie.github.io/apysc/array_comparison.html)':  # noqa
    '',

    '## value property API':
    '',

    '<!-- Docstring: apysc._type.array.Array.value -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface summary]** Get a current array value.<hr>\n\n**[Returns]**\n\n- `value`: list\n  - Current array value.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> arr: ap.Array = ap.Array([1, 2, 3])\n>>> arr.value = [4, 5, 6]\n>>> arr.value\n[4, 5, 6]\n```':  # noqa
    '',

    '<hr>\n\n**[References]**\n\n- [apysc fundamental data classes value interface](https://simon-ritchie.github.io/apysc/fundamental_data_classes_value_interface.html)':  # noqa
    '',

}



"""Class implementation for the custom event interface.
"""

from typing import Any
from typing import Callable
from typing import Dict
from typing import Optional
from typing import Union

from apysc._event.custom_event_type import CustomEventType
from apysc._event.event import Event
from apysc._event.handler import HandlerData
from apysc._html.debug_mode import add_debug_info_setting
from apysc._type.blank_object_interface import BlankObjectInterface

_CustomEventType = str
_HandlerName = str
_Handler = Callable[[Any, Any], None]


class CustomEventInterface(BlankObjectInterface):

    _custom_event_handlers: Dict[
        _CustomEventType,
        Dict[_HandlerName, HandlerData]]

    def _initialize_custom_event_handlers_if_not_initialized(
            self, *, custom_event_type_str: str) -> None:
        """
        Initialize the _custom_event_handlers data if this instance
        does not initialize it yet.

        Parameters
        ----------
        custom_event_type_str : str
            Target custom event type string.
        """
        if not hasattr(self, '_custom_event_handlers'):
            self._custom_event_handlers = {}
        if custom_event_type_str not in self._custom_event_handlers:
            self._custom_event_handlers[custom_event_type_str] = {}

    def _get_custom_event_type_str(
            self, *,
            custom_event_type: Union[CustomEventType, str]) -> str:
        """
        Get a custom event type string from a type value.

        Parameters
        ----------
        custom_event_type : CustomEventType or str
            Target custom event type or string.

        Returns
        -------
        custom_event_type_str : str
            A custom event type string.
        """
        if isinstance(custom_event_type, str):
            return custom_event_type
        custom_event_type_str: str = custom_event_type.value
        return custom_event_type_str

    def _set_custom_event_handler_data(
            self, *, handler: _Handler,
            custom_event_type_str: str,
            options: Optional[Any]) -> None:
        """
        Set a handler's data to the dictionary.

        Parameters
        ----------
        handler : _Handler
            Callable that this instance calls when its event's
            dispatching.
        custom_event_type_str : str
            A target custom event type's string.
        options : dict or None
            Optional arguments dictionary to be passed to a handler.
        """
        from apysc._event.handler import get_handler_name
        name: str = get_handler_name(handler=handler, instance=self)
        if options is None:
            options = {}
        self._custom_event_handlers[
            custom_event_type_str][name] = {  # type: ignore
                'handler': handler,
                'options': options,
        }

    def _unset_custom_event_handler_data(
            self, *, handler: _Handler,
            custom_event_type_str: str) -> None:
        """
        Unset a handler's data from the dictionary.

        Parameters
        ----------
        handler : _Handler
            Callable that this instance calls when its
            event's dispatching.
        custom_event_type_str : str
            A target custom event type's string.
        """
        from apysc._event.handler import get_handler_name
        if custom_event_type_str not in self._custom_event_handlers:
            return
        name: str = get_handler_name(handler=handler, instance=self)
        if name not in self._custom_event_handlers[custom_event_type_str]:
            return
        del self._custom_event_handlers[custom_event_type_str][name]

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='CustomEventInterface')
    def bind_custom_event(
            self, custom_event_type: Union[CustomEventType, str],
            handler: _Handler,
            e: Event,
            *,
            options: Optional[Any] = None,
            in_handler_head_expression: str = '') -> str:
        """
        Add a custom event listener setting.

        Parameters
        ----------
        custom_event_type : CustomEventType or str
            Target custom event type.
        handler : _Handler
            Callable that this instance calls when its
            event's dispatching.
        e : Event
            Event instance.
        options : dict or None, default None
            Optional arguments dictionary to be passed to a handler.
        in_handler_head_expression : str, default ''
            Optional expression to be added at the handler function's
            head position.

        Returns
        -------
        name : str
            Handler's name.

        References
        ----------
        - Bind and trigger the custom event document
            - https://simon-ritchie.github.io/apysc/bind_and_trigger_custom_event.html  # noqa
        - About the handler options' type document
            - https://simon-ritchie.github.io/apysc/about_handler_options_type.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> def on_custom_event(
        ...         e: ap.Event[ap.Rectangle], options: dict) -> None:
        ...     rectangle: ap.Rectangle = e.this
        ...     rectangle.fill_color = ap.String('#f0a')
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> e: ap.Event = ap.Event(this=rectangle)
        >>> _ = rectangle.bind_custom_event(
        ...     custom_event_type='my_custom_event',
        ...     handler=on_custom_event, e=e)
        >>> # Do something here and then trigger the custom event
        >>> rectangle.trigger_custom_event(
        ...     custom_event_type='my_custom_event')
        """
        from apysc._event.handler import append_handler_expression
        from apysc._event.handler import get_handler_name
        custom_event_type_str: str = self._get_custom_event_type_str(
            custom_event_type=custom_event_type)
        self._initialize_custom_event_handlers_if_not_initialized(
            custom_event_type_str=custom_event_type_str)
        self._set_custom_event_handler_data(
            handler=handler, custom_event_type_str=custom_event_type_str,
            options=options)
        name: str = get_handler_name(handler=handler, instance=self)
        self._append_custom_event_binding_expression(
            custom_event_type_str=custom_event_type_str, name=name)
        handler_data: HandlerData = \
            self._custom_event_handlers[custom_event_type_str][name]
        append_handler_expression(
            handler_data=handler_data, handler_name=name, e=e,
            in_handler_head_expression=in_handler_head_expression)
        return name

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='CustomEventInterface')
    def _append_custom_event_binding_expression(
            self, *, custom_event_type_str: str, name: str) -> None:
        """
        Append a custom event binding expression.

        Parameters
        ----------
        custom_event_type_str : str
            Target custom event type string.
        name : str
            Handler's name.
        """
        import apysc as ap
        blank_object_variable_name: str = self.blank_object_variable_name
        expression: str = (
            f'$({blank_object_variable_name})'
            f'.off("{custom_event_type_str}", {name});'
            f'\n$({blank_object_variable_name})'
            f'.on("{custom_event_type_str}", {name});'
        )
        ap.append_js_expression(expression=expression)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='CustomEventInterface')
    def trigger_custom_event(
            self, custom_event_type: Union[CustomEventType, str]) -> None:
        """
        Add a custom event trigger setting.

        Parameters
        ----------
        custom_event_type : CustomEventType or str
            Target custom event type.

        References
        ----------
        - Bind and trigger the custom event document
            - https://simon-ritchie.github.io/apysc/bind_and_trigger_custom_event.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> def on_custom_event(
        ...         e: ap.Event[ap.Rectangle], options: dict) -> None:
        ...     rectangle: ap.Rectangle = e.this
        ...     rectangle.fill_color = ap.String('#f0a')
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> e: ap.Event = ap.Event(this=rectangle)
        >>> _ = rectangle.bind_custom_event(
        ...     custom_event_type='my_custom_event',
        ...     handler=on_custom_event, e=e)
        >>> # Do something here and then trigger the custom event
        >>> rectangle.trigger_custom_event(
        ...     custom_event_type='my_custom_event')
        """
        import apysc as ap
        blank_object_variable_name: str = self.blank_object_variable_name
        custom_event_type_str: str = self._get_custom_event_type_str(
            custom_event_type=custom_event_type)
        expression: str = (
            f'$({blank_object_variable_name})'
            f'.trigger("{custom_event_type_str}");'
        )
        ap.append_js_expression(expression=expression)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='CustomEventInterface')
    def unbind_custom_event(
            self,
            custom_event_type: Union[CustomEventType, str],
            handler: _Handler) -> str:
        """
        Unbind (remove) a custom event listener setting.

        Parameters
        ----------
        custom_event_type : CustomEventType or str
            Target custom event type.
        handler : _Handler
            A handler for when the custom event is triggered.

        Returns
        -------
        name : str
            Handler's name.

        Examples
        --------
        >>> import apysc as ap
        >>> def on_custom_event(
        ...         e: ap.Event[ap.Rectangle], options: dict) -> None:
        ...     rectangle: ap.Rectangle = e.this
        ...     rectangle.fill_color = ap.String('#f0a')
        ...     rectangle.unbind_custom_event(
        ...         custom_event_type='my_custom_event',
        ...         handler=on_custom_event)
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> e: ap.Event = ap.Event(this=rectangle)
        >>> _ = rectangle.bind_custom_event(
        ...     custom_event_type='my_custom_event',
        ...     handler=on_custom_event, e=e)
        >>> # Do something here and then trigger the custom event
        >>> rectangle.trigger_custom_event(
        ...     custom_event_type='my_custom_event')
        """
        from apysc._event.handler import get_handler_name
        custom_event_type_str: str = self._get_custom_event_type_str(
            custom_event_type=custom_event_type)
        self._initialize_custom_event_handlers_if_not_initialized(
            custom_event_type_str=custom_event_type_str)
        self._unset_custom_event_handler_data(
            handler=handler, custom_event_type_str=custom_event_type_str)
        name: str = get_handler_name(handler=handler, instance=self)
        self._append_custom_event_unbinding_expression(
            custom_event_type_str=custom_event_type_str, name=name)
        return name

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='CustomEventInterface')
    def _append_custom_event_unbinding_expression(
            self, *, custom_event_type_str: str, name: str) -> None:
        """
        Add a custom event unbinding expression.

        Parameters
        ----------
        custom_event_type_str : str
            Target custom event type string.
        name : str
            Handler's name.
        """
        import apysc as ap
        expression: str = (
            f'$({self.blank_object_variable_name})'
            f'.off("{custom_event_type_str}", {name});'
        )
        ap.append_js_expression(expression=expression)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='CustomEventInterface')
    def unbind_custom_event_all(
            self, custom_event_type: Union[CustomEventType, str]) -> None:
        """
        Unbind (remove) custom event listener settings.

        Parameters
        ----------
        custom_event_type : CustomEventType or str
            Target custom event type.

        Examples
        --------
        >>> import apysc as ap
        >>> def on_custom_event(
        ...         e: ap.Event[ap.Rectangle], options: dict) -> None:
        ...     rectangle: ap.Rectangle = e.this
        ...     rectangle.fill_color = ap.String('#f0a')
        ...     rectangle.unbind_custom_event_all(
        ...         custom_event_type='my_custom_event')
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> e: ap.Event = ap.Event(this=rectangle)
        >>> _ = rectangle.bind_custom_event(
        ...     custom_event_type='my_custom_event',
        ...     handler=on_custom_event, e=e)
        >>> # Do something here and then trigger the custom event
        >>> rectangle.trigger_custom_event(
        ...     custom_event_type='my_custom_event')
        """
        import apysc as ap
        custom_event_type_str: str = self._get_custom_event_type_str(
            custom_event_type=custom_event_type)
        self._initialize_custom_event_handlers_if_not_initialized(
            custom_event_type_str=custom_event_type_str)
        self._custom_event_handlers[custom_event_type_str] = {}
        expression: str = (
            f'$({self.blank_object_variable_name})'
            f'.off("{custom_event_type_str}");'
        )
        ap.append_js_expression(expression=expression)



"""This module is for the translation mapping data of the
following document:

Document file: graphics_draw_dash_dotted_line.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# Graphics draw_dash_dotted_line interface':
    '',

    'This page explains the `Graphics` class `draw_dash_dotted_line` method interface.':  # noqa
    '',

    '## What interface is this?':
    '',

    '`draw_dash_dotted_line` interface draws the simple straight dash dotted-line (also called 1-dot chain line or long dashed short dashed line) graphics.\n\n This interface will ignore `dot_setting`, `dash_setting`, `round_dot_setting`, and `dash_dot_setting`.':  # noqa
    '',

    '## Basic usage':
    '',

    '`draw_dash_dotted_line` interface has basic coordinates arguments of `x_start`, `y_start`, `x_end` and `y_end`. That also has `dot_size` (the short dash size), `dash_size` (the long dash size) and `space_size` (the space size between the each dashes) arguments to determine line style.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nap.Stage(\n    background_color=\'#333\',\n    stage_width=250,\n    stage_height=130,\n    stage_elem_id=\'stage\')\nsprite: ap.Sprite = ap.Sprite()\n\n# Set 2-pixel dot size and 6-pixel dash size and draw the line.\nsprite.graphics.line_style(color=\'#0af\', thickness=5)\nsprite.graphics.draw_dash_dotted_line(\n    x_start=50, y_start=50, x_end=200, y_end=50,\n    dot_size=2, dash_size=6, space_size=5)\n\n# Set 5-pixel dot size and 10-pixel dash size and draw the line.\nsprite.graphics.draw_dash_dotted_line(\n    x_start=50, y_start=80, x_end=200, y_end=80,\n    dot_size=5, dash_size=10, space_size=5)\n\nap.save_overall_html(\n    dest_dir_path=\'graphics_draw_dash_dotted_line_basic_usage/\')\n```':  # noqa
    '',

    '<iframe src="static/graphics_draw_dash_dotted_line_basic_usage/index.html" width="250" height="130"></iframe>':  # noqa
    '',

    '## draw_dash_dotted_line API':
    '',

    '<!-- Docstring: apysc._display.graphics.Graphics.draw_dash_dotted_line -->\n\n<span class="inconspicuous-txt">Note: the document build script generates and updates this API document section automatically. Maybe this section is duplicated compared with previous sections.</span>\n\n**[Interface signature]** `draw_dash_dotted_line(self, x_start:Union[int, apysc._type.int.Int], y_start:Union[int, apysc._type.int.Int], x_end:Union[int, apysc._type.int.Int], y_end:Union[int, apysc._type.int.Int], dot_size:Union[int, apysc._type.int.Int], dash_size:Union[int, apysc._type.int.Int], space_size:Union[int, apysc._type.int.Int]) -> \'_line.Line\'`<hr>\n\n**[Interface summary]** Draw a dash-dotted (1-dot chain) line vector graphics.<hr>\n\n**[Parameters]**\n\n- `x_start`: Int or int\n  - Line start x-coordinate.\n- `y_start`: Int or int\n  - Line start y-coordinate.\n- `x_end`: Int or int\n  - Line end x-coordinate.\n- `y_end`: Int or int\n  - Line end y-coordinate.\n- `dot_size`: Int or int\n  - Dot size.\n- `dash_size`: Int or int\n  - Dash size.\n- `space_size`: Int or int\n  - Blank space size between dots and dashes.\n\n<hr>\n\n**[Returns]**\n\n- `line`: Line\n  - Created line graphics instance.\n\n<hr>\n\n**[Examples]**':  # noqa
    '',

    '```py\n>>> import apysc as ap\n>>> stage: ap.Stage = ap.Stage()\n>>> sprite: ap.Sprite = ap.Sprite()\n>>> sprite.graphics.line_style(color=\'#fff\', thickness=5)\n>>> line: ap.Line = sprite.graphics.draw_dash_dotted_line(\n...    x_start=50, y_start=50, x_end=150, y_end=50,\n...    dot_size=2, dash_size=5, space_size=3)\n>>> line.line_color\nString(\'#ffffff\')\n\n>>> line.line_dash_dot_setting.dot_size\nInt(2)\n\n>>> line.line_dash_dot_setting.dash_size\nInt(5)\n\n>>> line.line_dash_dot_setting.space_size\nInt(3)\n```':  # noqa
    '',

}



"""Class implementation for the path labels enum.
"""

from enum import Enum


class PathLabel(Enum):
    """
    Enum class for the path labels
    """
    MOVE_TO = 'M'
    LINE_TO = 'L'
    HORIZONTAL = 'H'
    VERTICAL = 'V'
    CLOSE = 'Z'
    BEZIER_2D = 'Q'
    BEZIER_2D_CONTINUAL = 'T'
    BEZIER_3D = 'C'
    BEZIER_3D_CONTINUAL = 'S'



"""This module is for the translation mapping data of the
following document:

Document file: string_comparison_operations.md
Language: jp
"""

from typing import Dict

MAPPING: Dict[str, str] = {

    '# String class comparison operations':
    '',

    'This page explains the `String` class comparison operations, like the `=`\\, `>=`\\.':  # noqa
    '',

    '## Comparison return value type':
    '',

    'Each `String` class comparison operation returns a `Boolean` value, not a Python built-in `bool` value.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstring_1: ap.String = ap.String(\'Hello\')\nresult: ap.Boolean = string_1 == \'Hello\'\nassert result\nassert isinstance(result, ap.Boolean)\n```':  # noqa
    '',

    '':
    '',

    '## Acceptable comparison right-side value types':
    '',

    'The `str` or `String` types of comparison other value (comparison right-side value) types are acceptable, for instance:':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstring_1: ap.String = ap.String(\'Hello\')\nresult: ap.Boolean = string_1 == \'Hello\'\nassert result\n```':  # noqa
    '',

    '':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstring_1: ap.String = ap.String(\'Hello\')\nstring_2: ap.String = ap.String(\'Hello\')\nresult: ap.Boolean = string_1 == string_2\nassert result\n```':  # noqa
    '',

    '':
    '',

    '## Equal comparison':
    '',

    'You can use the `==` operator for the equal comparison:':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstring_1: ap.String = ap.String(\'Hello\')\nresult: ap.Boolean = string_1 == \'Hello\'\nassert result\n```':  # noqa
    '',

    '':
    '',

    '## Not equal comparison':
    '',

    'You can use the `!=` operator for the not equal comparison:':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstring_1: ap.String = ap.String(\'Hello\')\nresult: ap.Boolean = string_1 != \'World\'\nassert result\n```':  # noqa
    '',

    '':
    '',

    '## Less than or greater than comparison':
    '',

    'You can use each less than, less than or equal, greater than, greater than equal comparison, with the `<`\\, `<=`\\, `>`\\, `>=` operators, like the Python built-in `str` value. Sometimes these operations are helpful to compare with the date (or date-time) string.':  # noqa
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstring_1: ap.String = ap.String(\'1970-01-05\')\nresult: ap.Boolean = string_1 < \'1970-01-06\'\nassert result\n```':  # noqa
    '',

    '':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstring_1: ap.String = ap.String(\'1970-01-05\')\nresult: ap.Boolean = string_1 <= \'1970-01-05\'\nassert result\n```':  # noqa
    '',

    '':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstring_1: ap.String = ap.String(\'1970-01-05\')\nresult: ap.Boolean = string_1 > \'1970-01-04\'\nassert result\n```':  # noqa
    '',

    '':
    '',

    '```py\n# runnable\nimport apysc as ap\n\nstring_1: ap.String = ap.String(\'1970-01-05\')\nresult: ap.Boolean = string_1 >= \'1970-01-05\'\nassert result\n```':  # noqa
    '',

}



"""Class implementation for click interface.
"""

from typing import Callable
from typing import Dict
from typing import Optional
from typing import TypeVar

from apysc._event.handler import HandlerData
from apysc._event.mouse_event import MouseEvent
from apysc._event.mouse_event_interface_base import MouseEventInterfaceBase
from apysc._html.debug_mode import add_debug_info_setting

_O = TypeVar('_O')
_Handler = Callable[[MouseEvent, _O], None]


class ClickInterface(MouseEventInterfaceBase):

    _click_handlers: Dict[str, HandlerData]

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='ClickInterface')
    def click(
            self, handler: _Handler[_O], *,
            options: Optional[_O] = None) -> str:
        """
        Add a click event listener setting.

        Parameters
        ----------
        handler : _Handler
            A callable would be called when clicking this instance.
        options : dict or None, default None
            Optional arguments dictionary to be passed to a handler.

        Returns
        -------
        name : str
            Handler's name.

        References
        ----------
        - Click interface document
            - https://simon-ritchie.github.io/apysc/click.html
        - About the handler options’ type document
            - https://simon-ritchie.github.io/apysc/about_handler_options_type.html  # noqa

        Examples
        --------
        >>> import apysc as ap
        >>> def on_click(
        ...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:
        ...     rectangle: ap.Rectangle = e.this
        ...     rectangle.x += 10
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> _ = rectangle.click(on_click)
        """
        import apysc as ap
        from apysc._event.handler import append_handler_expression
        from apysc._event.handler import get_handler_name
        from apysc._type.variable_name_interface import VariableNameInterface
        from apysc._validation.handler_options_validation import \
            validate_options_type
        from apysc._validation.variable_name_validation import \
            validate_variable_name_interface_type
        self_instance: VariableNameInterface = \
            validate_variable_name_interface_type(instance=self)
        validate_options_type(options=options)
        self._initialize_click_handlers_if_not_initialized()
        name: str = get_handler_name(handler=handler, instance=self)
        self._set_mouse_event_handler_data(
            handler=handler, handlers_dict=self._click_handlers,
            options=options)
        self._append_mouse_event_binding_expression(
            name=name, mouse_event_type=ap.MouseEventType.CLICK)
        e: ap.MouseEvent = ap.MouseEvent(this=self_instance)
        append_handler_expression(
            handler_data=self._click_handlers[name],
            handler_name=name, e=e)
        return name

    def _initialize_click_handlers_if_not_initialized(self) -> None:
        """
        Initialize _click_handlers attribute if it hasn't been
        initialized yet.
        """
        if hasattr(self, '_click_handlers'):
            return
        self._click_handlers = {}

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='ClickInterface')
    def unbind_click(self, handler: _Handler[_O]) -> None:
        """
        Unbind specified handler's click event.

        Parameters
        ----------
        handler : _Handler
            Unbinding target Callable.

        References
        ----------
        - Click interface document
            - https://simon-ritchie.github.io/apysc/click.html

        Examples
        --------
        >>> import apysc as ap
        >>> def on_click(
        ...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:
        ...     rectangle: ap.Rectangle = e.this
        ...     rectangle.fill_color = ap.String('#f0a')
        ...     rectangle.unbind_click(on_click)
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> _ = rectangle.click(on_click)
        """
        import apysc as ap
        self._initialize_click_handlers_if_not_initialized()
        self._unbind_mouse_event(
            handler=handler, mouse_event_type=ap.MouseEventType.CLICK,
            handlers_dict=self._click_handlers)

    @add_debug_info_setting(  # type: ignore[misc]
        module_name=__name__, class_name='ClickInterface')
    def unbind_click_all(self) -> None:
        """
        Unbind all click events.

        References
        ----------
        - Click interface document
            - https://simon-ritchie.github.io/apysc/click.html

        Examples
        --------
        >>> import apysc as ap
        >>> def on_click(
        ...         e: ap.MouseEvent[ap.Rectangle], options: dict) -> None:
        ...     rectangle: ap.Rectangle = e.this
        ...     rectangle.fill_color = ap.String('#f0a')
        ...     rectangle.unbind_click_all()
        >>> stage: ap.Stage = ap.Stage()
        >>> sprite: ap.Sprite = ap.Sprite()
        >>> sprite.graphics.begin_fill(color='#0af')
        >>> rectangle: ap.Rectangle = sprite.graphics.draw_rect(
        ...     x=50, y=50, width=50, height=50)
        >>> _ = rectangle.click(on_click)
        """
        import apysc as ap
        self._initialize_click_handlers_if_not_initialized()
        self._unbind_all_mouse_events(
            mouse_event_type=ap.MouseEventType.CLICK,
            handlers_dict=self._click_handlers)
